% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie berekent het aantal onafhankelijke patronen voor Robert Nottrot.
% De patronen worden gevormd door elementen in een rooster van m bij n vakjes, die twee waarden aan kunnen nemen.
% Vervolgens wordt dit rooster periodiek voortgezet over Z^2, oftewel, het is een patroon op Z_m x Z_n.
% Patronen worden hetzelfde genoemd als ze door rotatie, spiegeling of translatie in elkaar overgaan.
% Het aantal patronen wordt bepaald met Burnside's lemma, dus door het aantal vaste punten
% te bepalen voor alle elementen van de symmetriegroep.
% Voor m~=n heeft een deel van de patronen ook na draaiing d de goede periode; deze "dubbelperiodieke" patronen
% hebben zowel horizontaal als verticaal zowel periode m als periode n, dus minimale periode is dan gcd(m,n) in beide richtingen.
% Die elementen zijn dan in feite permutaties van de m bij n vakjes, samen met het teken van de draaiing en spiegeling,
% je moet immers weten of het ene element door de draaispiegeling wordt omgezet in de andere of juist niet.
% Zie ook de discussie van de verschillende mogelijkheden voor de elementen.
% Het aantal vaste punten wordt bepaald door het aantal cykels in bovengenoemde permutatie.
% Elke cykel kan namelijk twee varianten hebben voor wat betreft de elementen, maar is verder bepaald.
% Daarbij moet nog opgemerkt worden dat het voorkomen van een oneven cykel bij een negatieve draaispiegeling
% het aantal mogelijke symmetrische patronen direct tot nul reduceert.
%
% De INVOER-parameter vulling met mogelijke waarden 'x+', '-|', 'sz', '\/', '<>' of 'MW' bepaalt wat de symmetrie is
% van de elementen waaruit de patronen worden opgebouwd, oftewel wat het teken is van s en d.
% Namelijk:
%   - 'x+': 'x' en '+' blijven beide behouden bij spiegeling s en draaiing d, dus sgns = 0 = sgnd.
%   - '-|': '-' en '|' blijven behouden bij spiegeling s, maar gaan in elkaar over bij draaiing d, dus sgns=0, sgnd=1.
%   - 'sz': Hierbij staat 's' eigenlijk voor \-  /  en 'z' voor \  -/  Deze zijn beide symmetrisch onder draaiing d,
%                                             \ /|              |\ /   maar gaan in elkaar over onder spiegeling s.
%                                              X                  X    Dus sgns = 1, sgnd = 0.
%                                            |/ \                / \|
%                                            /  -\              /-  \
%   - '\/': Dit is de DEFAULT-waarde waar de oorspronkelijke vraag van Robert Nottrot over ging.
%           '\' en '/' gaan zowel onder spiegeling s als onder draaiing d in elkaar over, dus sgns = 1 = sgnd.
%   - '<>': Bij deze locale elementen mag draaiing d niet gebruikt worden, omdat je dan meer dan twee elementen
%           zou krijgen ('v' en '^' erbij) en dat vergt nog meer programmeer-veranderingen. Puntspiegeling d^2
%           voert deze elementen in elkaar over, net als spiegeling s, dus sgns = 1 = sgndd.
%   - 'MW': Ook hierbij mag draaiing d niet gebruikt worden, alleen puntspiegeling d^2, er geldt sgns = 0, sgndd = 1.
%               Deze laatste twee zullen overigens vaak hetzelfde resultaat geven!
%
% De INVOER-parameter trafos wordt gebruikt om eventueel bepaalde transformatie's uit de groep te weren, als volgt:
%   - trafos(1:2) bepaalt de voortbrenger voor in principe de verticale translatie's, zoals
%   - trafos(3:4) bepaalt wat de voortbrenger voor in principe de horizontale translatie's is:
%       [0 0] = deze translatie mag niet gebruikt worden.
%       [i j] = x^i y^j: i stappen naar boven (m-i naar beneden) en j naar rechs (n-j naar links) is de voortbrenger.
%   - trafos(5) bepaalt welke spiegeling als voortbrenger wordt gebruikt:
%       0 = geen spiegelingen
%       1 = spiegeling s, in verticale as
%       2 = spiegeling sd^2, in horizontale as
%       3 = spiegeling sd^3 in stijgende diagonaal (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%       4 = spiegeling sd in dalende diagonaal (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%     Merk op dat het geen zin heeft om twee verschillende spiegelingen als voortbrengers op te voeren,
%     omdat dat een draaiing is die met vulling(6) wordt bepaald. Verder maakt het niet uit welke van
%     de spiegelingen als voortbrenger wordt gebruikt als je ook alle draaiingen toelaat. Met dien verstande
%     dat het kiezen van sd^3 of sd voor m~=n ertoe leidt dat spiegelen alleen op dubbelperiodieke wordt toegepast.
%   - trafos(6) bepaalt welke draaiing als voortbrenger wordt gebruikt:
%       0 = draaiingen worden niet toegestaan
%       1 = draaiing kwartslag rechtsom (voor m~=n alleen op dubbelperiodieke, op andere d^2, puntspiegeling)
%       2 = draaiing d^2 over 180 graden, puntspiegeling (ook als m~=n dan geeft dit d^2, puntspiegeling)
%       3 = draaiing d^3 kwartslag linksom (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%              LET OP: Voor m~=n geeft dit programma met voortbrenger d^3 dus andere resultaten dan met voortbrenger d,
%              ook al is dat niet wat je zou verwachten, omdat er nu anders op de niet-dubbelperiodieke wordt gewerkt!
%   - trafos(7) bepaalt of de flip-transformatie meedoet:
%       0 = de transformatie f doet niet mee
%       1 = de transformatie f mag gebruikt worden 
%           (deze verplaatst niets maar verandert elk element in zijn tegenhanger, is dus e met negatief teken).
% Om een en ander op een meer intuitieve manier in te voeren mag trafos ook een string zijn
% waarin het aantal keren dat x,Y,X,y,s,d en f voorkomen de getallen trafos(1:7) vervangen.
% (x,Y worden dus bij elkaar genomen tot trafos(1:2) en X,y worden samen genomen tot trafos(3:4)).
% Voor spiegelingen is dit wellicht enigszins tegen-intuitief: bijvoorbeeld sd^3 moet als 'sss' gecodeerd worden,
% terwijl 'sddd' geinterpreteerd zou worden als trafos(5:6) = [1 3].
%
% Tenslotte is de array telling een 1x5-array die bepaalt op welke manier de patronen meetellen,
% dit is ten behoeve van het bepalen van de correctie voor niet meenemen van d voor m~=n.
% Dan moet je het aantal dubbelperiodieke patronen soms aftrekken om de niet dubbelperiodieke over te houden.
% voor de gebruiker is dit niet echt van belang, want je zult het niet zelf gebruiken.
% Er wordt een factor telling(xysdf)^i toegevoegd als "weging" van xysdf^i. Dit wordt vooral toegepast met telling(4)=-1.
% Het kan overigens ook allemaal zonder telling, maar dan moet je twee keer de groep voor gcd(m,n) x gcd(m,n) doorlopen!

function patronen = aantalpatronen(m,n, vulling, trafos, telling)

% checken dat m,n twee gehele getallen zijn.
if prod([size(m), size(n)]) ~= 1 | ceil([m,n]) ~= [m,n] | m <= 0 | n <=0
    error('m and n must be positive integers!');
end
% DEFAULT-waarde voor telling, trafos en vulling
if nargin < 5
    telling = [1 1 1 1 1];
if nargin < 4
    trafos = [1 0 0 1 1 1 0];
if nargin < 3
    vulling = '\/';
end;end;end
if isempty(vulling) vulling = '\/'; end % vulling = '' is placeholder.
if isempty(trafos) trafos = [1 0 0 1 1 1 0]; end % trafos = [] is placeholder.

% vulling in een van de zes standaard-vormen brengen.
switch vulling
case {'\/','\','/','/\','\\','//'}
    vulling = '\/';
case {'-|','-','|','|-','--','||'}
    vulling = '-|';
case {'x+','x','+','+x','xx','++'}
    vulling = 'x+';
case {'sz','s','z','zs','ss','zz'}
    vulling = 'sz';
case {'<>','<','>','><','<<','>>'}
    vulling = '<>';
case {'MW','M','W','WM','MM','WW'}
    vulling = 'MW';
otherwise
    error(['vulling = ''' vulling ''' could not be interpreted as code for local symmetry!'])
end % switch vulling
% trafos in juiste vorm brengen
trafos = reshape(trafos,1,prod(size(trafos)));
if ischar(trafos)
    invoertrafos=trafos;
    trafos = zeros(1,7);
    trafos(1) = length(find(invoertrafos=='x'));
    trafos(2) = length(find(invoertrafos=='Y'));
    trafos(3) = length(find(invoertrafos=='X'));
    trafos(4) = length(find(invoertrafos=='y'));
    trafos(5) = length(find(invoertrafos=='s'));
    trafos(6) = length(find(invoertrafos=='d'));
    trafos(7) = length(find(invoertrafos=='f'));
    clear invoertrafos;
end % if trafos string -> getallen
if length(trafos) < 7 defaulttrafos = [1 0 0 1 1 1 0]; trafos = [trafos defaulttrafos(length(trafos)+1,end)]; end % trafos zo nodig aanvullen met DEFAULT-waarden.
trafos = floor(trafos(1:7));
if ismember(vulling(1:2), {'<>', 'MW'})
    if ismember(mod(trafos(5),5),[3 4]) trafos(5) = 0; end % Voor vulling = '<>' of 'MW' wordt geen diagonale spiegeling toegestaan.
	if mod(trafos(6),4) trafos(6) = 2; end % Voor vulling = '<>' of 'MW' wordt alleen puntspiegeling d^2 toegestaan.
end
% Vanaf nu is vulling = '\/', '-|', 'x+', 'sz', '<>' of 'MW' en trafos is een 1x7 array met integers.
% Waarbij trafos(5)=0,1,2 en trafos(6)=0,2 in het geval dat vulling = '<>' of 'MW'.
telling = reshape(telling,1,prod(size(telling)));
if ~isnumeric(telling) | length(telling) ~= 5 error('Invoer telling moet een numerieke 1x5-array zijn!'); end


% Maken van de voortbrengende operaties.
e = reshape([1:m*n],m,n);
% ----- x ------
% De voortbrenger x = x^trafos(1) y^trafos(2), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordex.
mod(trafos(1),m); x = [e(ans+1:end,:); e(1:ans,:)]; ordex = m/gcd(m,ans);
mod(trafos(2),n); Y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordeY = n/gcd(n,ans);
x = Y(x); ordex = lcm(ordex,ordeY); clear Y ordeY;
% ----- y -----
% De voortbrenger y = x^trafos(3) y^trafos(4), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordey.
mod(trafos(3),m); X = [e(ans+1:end,:); e(1:ans,:)]; ordeX = m/gcd(m,ans);
mod(trafos(4),n); y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordey = n/gcd(n,ans);
y = y(X); ordey = lcm(ordeX,ordey); clear X ordeX;
% ----- xysymm -----
if m~=n
    noxysymm = 1; % Geen xy-symmetrie
elseif mod([trafos(1)-trafos(4) trafos(1)+trafos(4)], m) % als trafos(1) ~= +- trafos(4) modulo m(==n)
    noxysymm = 1; % Geen xy-symmetrie
elseif mod([trafos(2)-trafos(3) trafos(2)+trafos(3)], m) % als trafos(2) ~= +- trafos(3) modulo m(==n)
    noxysymm = 1; % Geen xy-symmetrie
elseif ismember(vulling(1:2), {'<>', 'MW'}) % als de locale elementen verschil maken tussen horizontaal en verticaal
    noxysymm = 1; % Geen xy-symmetrie
else
    % Maar als m==n en zowel de x in x correspondeert met de y in y en de Y in x met de X in y:
    noxysymm = 0; % Er is xy-symmetrie
end % xy-symmetrie ?
% Voor m==n, trafos(1)==trafos(4) en trafos(2)==trafos(3) kunnen we de equivalentie van x <-> y gebruiken.
	% Zo ook als als de correspondentie slechts na omkering van boven/onder, links/rechts aanwezig is.
    % Merk op dat er misschien wel meer gevallen zijn waarbij dit in principe zou kunnen, zolang dezelfde ondergroep
    % wordt voortgebracht. Daarbij is het echter lastig omdat de volgorde waarin de voortgebrachte groep wordt
    % doorlopen niet overeen hoeft te komen. Het kan daarom voor de snelheid van de routine net iets beter zijn
    % als je er zelf voor zorgt dat je de kleinste voortbrengers kiest.
if [mod(trafos(1),m)*mod(trafos(3),m) mod(trafos(2),n)*mod(trafos(4),n)]==0 xyonafh = 1; else xyonafh = 0; end
    % Als noch x, noch y in beide voortbrengers tegelijk voorkomt, dan zijn de voortbrengers onafhankelijk.
    % Anders kun je niet meer zeggen dat x^i y^ordey-j equivalent is met x^i y^j.
    % In feite heeft dit weer te maken met een wijziging in de volgorde waarin elementen doorlopen worden.
    % Slim kiezen van de voortbrengers kan dus mogelijk weer versnelling opleveren.
    % Overigens verwacht ik dat er meer situaties zijn waarin je xyonafh op 1 zou mogen zetten, bijvoorbeeld
    % als trafos(1:2) loodrecht staat op trafos(3:4). Als m~=n dan is dat in ieder geval niet waar,
    % maar voor m==n lijkt het mij wel in orde. Ik kan dat momenteel echter nog niet bewijzen,
    % dus maak ik er nog maar geen gebruik van. Waar het volgens mij om moet gaan is de vraag of
    % x^i y^j == e kan gebeuren voor i,j ~= ordex,ordey (of veelvouden). Oftewel, zijn er niet-triviale i,j
    % waarvoor mod(i*trafos(1)+j*trafos(3), m)==0 en mod(i*trafos(2)+j*trafos(4), n)==0.
% ----- s -----
% De voortbrengende spiegeling, met orde ordes en teken sgns. (in de switch trafos(5) voor het geval vulling == '\/')
persd = 0; % Deze bepaalt of er nog extra rekening gehouden moet worden met draaiingen en spiegeling op dubbel-periodieke patronen.
switch mod(trafos(5),5)
case 0 % geen spiegelingen.
    s = e; ordes = 1; sgns = 0;
case 1 % spiegeling in de verticale as.
    s = e(:,end:-1:1); ordes = 2; sgns = 1;
case 2 % spiegeling in de horizontale as.
    s = e(end:-1:1,:); ordes = 2; sgns = 1;
case 3 % spiegeling in de stijgende diagonaal.
    if m==n
        s = e(end:-1:1,end:-1:1)'; ordes = 2; sgns = 0;
    else
        % Als m~=n wordt sd^3 uitgeschakeld maar de flag voor correctie aangezet.
        s = e; ordes = 1; sgns = 0; persd = persd+1;
    end
case 4 % spiegeling in de dalende diagonaal.
    if m==n
        s = e'; ordes = 2; sgns = 0;
    else
        % Als m~=n wordt sd uitgeschakeld maar de flag voor correctie aangezet.
        s = e; ordes = 1; sgns = 0; persd = persd+1;
    end
otherwise
    error('trafos(5) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if vulling == '-|' sgns = (ordes-1)*mod(sgns+1,2); end % voor vulling == '-|' zijn de locale spiegeleigenschappen precies andersom.
if vulling == 'x+' sgns = 0; end         % voor vulling == 'x+' zorgt spiegelen nooit voor een omkering.
if vulling == 'sz' sgns = (ordes-1); end % voor vulling == 'sz' zorgt spiegelen altijd voor een omkering.
if vulling == '<>' & mod(trafos(5),5) == 2 sgns = 0; end % Voor vulling = '<>' is alleen voor horizontale spiegeling het teken anders dan voor '\/'.
if vulling == 'MW' & mod(trafos(5),5) == 1 sgns = 0; end % Voor vulling = 'MW' is alleen voor verticale spiegeling het teken anders dan voor '\/'.
% ----- d -----
% De voortbrenger d = d^trafos(6), met d in rl draaiing kwartslag rechtsom. De orde is orded, het teken is sgnd.
switch mod(trafos(6),4)
case 0 % geen draaiingen.
    d = e; orded = 1; sgnd = 0;
case 1 % draaiing kwartslag rechtsom.
    if m==n
        d = e(end:-1:1,:)'; orded = 4; sgnd = 1;
    else
        % Als m~=n wordt d vervangen door d^2 maar de flag voor correctie aangezet.
        d = e(end:-1:1,end:-1:1); orded = 2; sgnd = 0; persd=persd+2;
    end
case 2 % draaiing kwartslag rechtsom.
    d = e(end:-1:1,end:-1:1); orded = 2; sgnd = 0;
case 3 % draaiing kwartslag linksom.
    if m==n
        d = e(:,end:-1:1)'; orded = 4; sgnd = 1;
    else
        % Als m~=n wordt d^3 vervangen door e maar de flag voor correctie aangezet.
        d = e; orded = 2; sgnd = 0; persd=persd+4;
    end
otherwise
    error('trafos(6) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if ismember(vulling, {'x+','sz'}) sgnd = 0; end % voor vulling == 'x+' of 'sz' zorgt draaien nooit voor een omkering.
if ismember(vulling, {'<>','MW'}) & (orded-1) sgnd = 1; end % Bij vulling = '<>' of 'MW' zorgt puntspiegeling voor een omkering.
% ----- f -----
% Wel of niet de flip-transformatie
if mod(trafos(7),2) ordef = 2; else ordef = 1; end
 % Je zou ook sgnf=1 resp. 0 kunnen definieren, maar dat is niet nodig omdat je toch al weet dat f~=0 alleen als sgnf=1


% ========================================================
% Nu op een slimme manier alle groepselementen langslopen.
patronen = zeros([ordex+1,ordey+1,ordes,orded,ordef]); % initialisatie van de array die de vaste punten telt.
xi = e; % initialisatie van de x-translatie waar we mee bezig zijn.
for i=0:floor(ordex/2)
yj = e; % initialisatie van de y-translatie waar we mee bezig zijn.
for j=0:(i+(floor(ordey/2)-i)*noxysymm-ordey+1)*xyonafh + ordey-1
        % Als m==n, trafos(1)==trafos(4) en trafos(2)==trafos(3) kunnen we de equivalentie van x <-> y gebruiken.
        % en dus hoeven alleen de translaties x^i y^j met j<=i doorgerekend te worden.
        % Net zoals we de equivalentie van x^i <-> x^(ordex-i) altijd gebruiken
        % en y^i <-> y^(ordey-i) alleen gebruiken als de voortbrengers x en y onafhankelijk zijn.
        % Voor m~=n of als de voorbrengers niet equivalent zijn, moeten wel alle translaties doorgerekend worden,
        % Bij afhankelijkheid tussen x en y kun je helemaal niet meer slim over y lopen.
    % We lopen de xi(yj(draaispiegelingen)) af voor deze translatie
    sk = e;
    for k=0:ordes-1
    dl = e; % initialisatie van de draaiing waar we mee bezig zijn.
    for l=0:orded-1
        % Construeren van de permutatie en de cykellengtes bepalen.
        p = yj(xi);
        p = sk(p);
        p = dl(p);
        banen = cykellengtes(p);
        for f=0:ordef-1
            if mod(k*sgns+l*sgnd+f,2)==0 % het teken van de draaispiegeling s^k d^l en een eventuele flip
                                         % Merk op dat f=f*sgnf omdat f~=0 alleen als sgnf=1
                % element-behoudende draaispiegeling
                patronen(i+1,j+1,k+1,l+1,f+1) = pow2(length(banen));
            else
                % element-inverterende draaispiegeling
                if ~mod(banen,2)
                    % alle cykels zijn even (er zijn er geen die oneven zijn)
                    patronen(i+1,j+1,k+1,l+1,f+1) = pow2(length(banen));
                else
                    % er is een oneven cykel
                    patronen(i+1,j+1,k+1,l+1,f+1) = 0;
                end
            end
        end % flip-loop
    dl = dl(d);
    end % d-loop
    sk = sk(s);
    end % s-loop

    % Tenslotte de teller 'patronen' vullen voor de isomorfe translaties
    % (soms is het gewoon hetzelfde element dat nog een keer met dezelfde inhoud wordt gevuld,
    %  maar dat is niet erg, want dan mocht hij gewoon niet dubbel tellen)
    patronen(ordex-i+1,j+1, :,:,:) = patronen(i+1,j+1, :,:,:);

    % Als de voortbrengers x en y onafhankelijk zijn, dan kunnen we nog wat equivalenties gebruiken:
    if xyonafh == 1
        patronen(i+1,ordey-j+1, :,:,:) = patronen(i+1,j+1, :,:,:);
        patronen(ordex-i+1,ordey-j+1, :,:,:) = patronen(i+1,j+1, :,:,:);

        % Als m==n, trafos(1)==+-trafos(4) en trafos(2)==+-trafos(3) dan gebruiken we ook nog de equivalentie x <-> y
        if noxysymm == 0
            patronen(j+1,i+1, :,:,:) = patronen(i+1,j+1, :,:,:);
            patronen(j+1,ordex-i+1, :,:,:) = patronen(i+1,j+1, :,:,:);
            patronen(ordey-j+1,i+1, :,:,:) = patronen(i+1,j+1, :,:,:);
            patronen(ordey-j+1,ordex-i+1, :,:,:) = patronen(i+1,j+1, :,:,:);
        end
	end
yj = yj(y);
end
xi = xi(x);
end % translatie-loop

% Nu is patronen dus gevuld met alle aantallen vaste punten. We hoeven alleen nog het gemiddelde te nemen.
patronen = patronen(1:ordex, 1:ordey, :,:,:); % in de laatste rijen/kolommen staan opnieuw de patronen voor e,
                                      % maar dat staat ook al in de eerste rij/kolom en moet niet dubbel geteld worden.
                                      % (dit komt doordat x^ordex = e, y^ordey = e)

% Correctie van de telling.
% N.B. Dit zou misschien minder tijd (of in ieder geval minder geheugen) kosten als je het in de loop al doet.
%   Aan de andere kant moet je dan telkens in de loop extra rekenen en nu gebruik je wellicht matlab's snelheid met matrices.
% Eerst 5-D matrices even groot als patronen maken met de exponenten van x,y,s,d en f.
[i,j,k,l,f] = ndgrid(0:ordex-1,0:ordey-1,0:ordes-1,0:orded-1,0:ordef-1);
% bijbehorende factoren telling^exp verrekenen.
patronen = telling(1).^i .* patronen;
patronen = telling(2).^j .* patronen;
patronen = telling(3).^k .* patronen;
patronen = telling(4).^l .* patronen;
patronen = telling(5).^f .* patronen;
% En dan tenslotte het gemiddeld aantal vaste punten bepalen.
patronen = mean(mean(mean(mean(mean(patronen)))));

% Nu eventueel nog corrigeren op draaiingen en spiegelingen op dubbelperiodieke patronen.
switch persd
case 0
    % Geen correctie aangevraagd
case 1
    % Er moet worden gecorrigeerd voor spiegeling op dubbelperiodieke patronen.
    p = gcd(m,n); % de dubbelperiodieke periode
    patronen = patronen - aantalpatronen(p,p, vulling, trafos, [1 1 -1 1 1]);
case 2
    % Er moet worden gecorrigeerd voor draaiing op dubbelperiodieke patronen.
    p = gcd(m,n); % de dubbelperiodieke periode
    patronen = patronen - aantalpatronen(p,p, vulling, trafos, [1 1 1 -1 1]);
case 3
    % Er moet worden gecorrigeerd voor zowel draaiing als spiegeling op dubbelperiodieke patronen.
    p = gcd(m,n); % de dubbelperiodieke periode
    patronen = patronen + aantalpatronen(p,p, vulling, trafos); % de dubbel-periodieke patronen erbij optellen.
    trafos(5:6) = [0 2]; % De spiegelingen en kwartslag-draaiingen er uit halen.
    patronen = patronen - aantalpatronen(p,p, vulling, trafos); % De dubbelperiodieke vaste punten op groepselementen,
    % die wel ook op niet-dubbelperiodieke mochten werken worden er weer afgetrokken. Die tel je anders namelijk dubbel.
case 4
    % Er moet gecorrigeerd worden voor draaiing, maar nu is ook d^2 uitgeschakeld.
    p = gcd(m,n); % de dubbelperiodieke periode
    patronen = patronen + aantalpatronen(p,p, vulling, trafos); % de dubbel-periodieke patronen erbij optellen.
    trafos(6) = 0; % Alle draaiingen er uit halen.
    patronen = patronen - aantalpatronen(p,p, vulling, trafos); % De dubbelperiodieke vaste punten op groepselementen,
    % die wel ook op niet-dubbelperiodieke mochten werken worden er weer afgetrokken. Die tel je anders namelijk dubbel.
case 5
    % Er moet gecorrigeerd worden voor zowel draaiing als spiegeling, maar nu is ook d^2 uitgeschakeld.
    p = gcd(m,n); % de dubbelperiodieke periode
    patronen = patronen + aantalpatronen(p,p, vulling, trafos); % de dubbel-periodieke patronen erbij optellen.
    trafos(5:6) = [0 0]; % De spiegelingen en draaiingen er uit halen.
    patronen = patronen - aantalpatronen(p,p, vulling, trafos); % De dubbelperiodieke vaste punten op groepselementen,
    % die wel ook op niet-dubbelperiodieke mochten werken worden er weer afgetrokken. Die tel je anders namelijk dubbel.
otherwise
    error(['De correctie-flag persd = ' int2str(persd) ' heeft een waarde die niet zou mogen voorkomen!'])
end