% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie bepaalt uit een opgegeven stel patronen q de banen van onafhankelijke patronen.
% De patronen worden gevormd door elementen in een rooster van m bij n vakjes, die twee waarden aan kunnen nemen.
% Vervolgens wordt dit rooster periodiek voortgezet over Z^2, oftewel, het is een patroon op Z_m x Z_n.
% Patronen worden hetzelfde genoemd als ze door rotatie, spiegeling of translatie in elkaar overgaan.
% Patronen worden gerepresenteerd door een getal. De binaire ontwikkeling ervan staat voor het patroon:
% 0 <-> '\','-','x','s','<' of 'M', 1 <-> '/','|','+','z','>' of 'W',
% gegeven een getal q, is p = dec2bin(q,m*n) de binaire ontwikkeling en p(e)=reshape(p,m,n) het bijbehorende patroon.
% De UITVOER q is de vector waarbij q(r) is vervangen door de eerste representant uit dezelfde baan.
% DEFAULT is om dit uit alle 2^(m*n) op het m x n rooster te doen, maar als je zelf een array van patroongetallen
%   q meegeeft, dan worden alleen de banen daaruit op hun equivalentie gecheckt.
%
% De INVOER-parameter vulling met mogelijke waarden 'x+', '-|', 'sz', '\/', '<>' of 'MW' bepaalt wat de symmetrie is
% van de elementen waaruit de patronen worden opgebouwd, oftewel wat het teken is van s en d.
% Namelijk:
%   - 'x+': 'x' en '+' blijven beide behouden bij spiegeling s en draaiing d, dus sgns = 0 = sgnd.
%   - '-|': '-' en '|' blijven behouden bij spiegeling s, maar gaan in elkaar over bij draaiing d, dus sgns=0, sgnd=1.
%   - 'sz': Hierbij staat 's' eigenlijk voor \-  /  en 'z' voor \  -/  Deze zijn beide symmetrisch onder draaiing d,
%                                             \ /|              |\ /   maar gaan in elkaar over onder spiegeling s.
%                                              X                  X    Dus sgns = 1, sgnd = 0.
%                                            |/ \                / \|
%                                            /  -\              /-  \
%   - '\/': Dit is de DEFAULT-waarde waar de oorspronkelijke vraag van Robert Nottrot over ging.
%           '\' en '/' gaan zowel onder spiegeling s als onder draaiing d in elkaar over, dus sgns = 1 = sgnd.
%   - '<>': Bij deze locale elementen mag draaiing d niet gebruikt worden, omdat je dan meer dan twee elementen
%           zou krijgen ('v' en '^' erbij) en dat vergt nog meer programmeer-veranderingen. Puntspiegeling d^2
%           voert deze elementen in elkaar over, net als spiegeling s, dus sgns = 1 = sgndd.
%   - 'MW': Ook hierbij mag draaiing d niet gebruikt worden, alleen puntspiegeling d^2, er geldt sgns = 0, sgndd = 1.
%               Deze laatste twee zullen overigens vaak hetzelfde resultaat geven!
%
% De INVOER-parameter trafos wordt gebruikt om eventueel bepaalde transformatie's uit de groep te weren, als volgt:
%   - trafos(1:2) bepaalt de voortbrenger voor in principe de verticale translatie's, zoals
%   - trafos(3:4) bepaalt wat de voortbrenger voor in principe de horizontale translatie's is:
%       [0 0] = deze translatie mag niet gebruikt worden.
%       [i j] = x^i y^j: i stappen naar boven (m-i naar beneden) en j naar rechs (n-j naar links) is de voortbrenger.
%   - trafos(5) bepaalt welke spiegeling als voortbrenger wordt gebruikt:
%       0 = geen spiegelingen
%       1 = spiegeling s, in verticale as
%       2 = spiegeling sd^2, in horizontale as
%       3 = spiegeling sd^3 in stijgende diagonaal (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%       4 = spiegeling sd in dalende diagonaal (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%     Merk op dat het geen zin heeft om twee verschillende spiegelingen als voortbrengers op te voeren,
%     omdat dat een draaiing is die met vulling(6) wordt bepaald. Verder maakt het niet uit welke van
%     de spiegelingen als voortbrenger wordt gebruikt als je ook alle draaiingen toelaat. Met dien verstande
%     dat het kiezen van sd^3 of sd voor m~=n ertoe leidt dat spiegelen alleen op dubbelperiodieke wordt toegepast.
%   - trafos(6) bepaalt welke draaiing als voortbrenger wordt gebruikt:
%       0 = draaiingen worden niet toegestaan
%       1 = draaiing kwartslag rechtsom (voor m~=n alleen op dubbelperiodieke, op andere d^2, puntspiegeling)
%       2 = draaiing d^2 over 180 graden, puntspiegeling (ook als m~=n dan geeft dit d^2, puntspiegeling)
%       3 = draaiing d^3 kwartslag linksom (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%              LET OP: Voor m~=n geeft dit programma met voortbrenger d^3 dus andere resultaten dan met voortbrenger d,
%              ook al is dat niet wat je zou verwachten, omdat er nu anders op de niet-dubbelperiodieke wordt gewerkt!
%   - trafos(7) bepaalt of de flip-transformatie meedoet:
%       0 = de transformatie f doet niet mee
%       1 = de transformatie f mag gebruikt worden 
%           (deze verplaatst niets maar verandert elk element in zijn tegenhanger, is dus e met negatief teken).
% Om een en ander op een meer intuitieve manier in te voeren mag trafos ook een string zijn
% waarin het aantal keren dat x,Y,X,y,s,d en f voorkomen de getallen trafos(1:7) vervangen.
% (x,Y worden dus bij elkaar genomen tot trafos(1:2) en X,y worden samen genomen tot trafos(3:4)).
% Voor spiegelingen is dit wellicht enigszins tegen-intuitief: bijvoorbeeld sd^3 moet als 'sss' gecodeerd worden,
% terwijl 'sddd' geinterpreteerd zou worden als trafos(5:6) = [1 3].

function q = bepaalbanen(m,n, vulling, trafos, q)

% checken dat m,n twee gehele getallen zijn.
if prod([size(m), size(n)]) ~= 1 | ceil([m,n]) ~= [m,n] | m <= 0 | n <=0
    error('m and n must be positive integers!');
end
% DEFAULT-waarde voor trafos en vulling
if nargin < 5
    % De rij getallen die de patronen bepalen initialiseren.
	q = [0:2^(m*n)-1]';
if nargin < 4
    trafos = [1 0 0 1 1 1 0];
if nargin < 3
    vulling = '\/';
end;end;end
if isempty(vulling) vulling = '\/'; end % vulling = '' is placeholder.
if isempty(trafos) trafos = [1 0 0 1 1 1 0]; end % trafos = [] is placeholder.

% Checken dat q een patroon representeert
if ceil(q) ~= q
    error('Patterns should be represented by integers q!');
end
% patroongetallen modulo maximum getal.
q = mod(q,2^(m*n));

% vulling in een van de zes standaard-vormen brengen.
switch vulling
case {'\/','\','/','/\','\\','//'}
    vulling = '\/';
case {'-|','-','|','|-','--','||'}
    vulling = '-|';
case {'x+','x','+','+x','xx','++'}
    vulling = 'x+';
case {'sz','s','z','zs','ss','zz'}
    vulling = 'sz';
case {'<>','<','>','><','<<','>>'}
    vulling = '<>';
case {'MW','M','W','WM','MM','WW'}
    vulling = 'MW';
otherwise
    error(['vulling = ''' vulling ''' could not be interpreted as code for local symmetry!'])
end % switch vulling
% trafos in juiste vorm brengen
trafos = reshape(trafos,1,prod(size(trafos)));
if ischar(trafos)
    invoertrafos=trafos;
    trafos = zeros(1,7);
    trafos(1) = length(find(invoertrafos=='x'));
    trafos(2) = length(find(invoertrafos=='Y'));
    trafos(3) = length(find(invoertrafos=='X'));
    trafos(4) = length(find(invoertrafos=='y'));
    trafos(5) = length(find(invoertrafos=='s'));
    trafos(6) = length(find(invoertrafos=='d'));
    trafos(7) = length(find(invoertrafos=='f'));
    clear invoertrafos;
end % if trafos string -> getallen
if length(trafos) < 7 defaulttrafos = [1 0 0 1 1 1 0]; trafos = [trafos defaulttrafos(length(trafos)+1,end)]; end % trafos zo nodig aanvullen met DEFAULT-waarden.
trafos = floor(trafos(1:7));
if ismember(vulling(1:2), {'<>', 'MW'})
    if ismember(mod(trafos(5),5),[3 4]) trafos(5) = 0; end % Voor vulling = '<>' of 'MW' wordt geen diagonale spiegeling toegestaan.
	if mod(trafos(6),4) trafos(6) = 2; end % Voor vulling = '<>' of 'MW' wordt alleen puntspiegeling d^2 toegestaan.
end
% Vanaf nu is vulling = '\/', '-|', 'x+', 'sz', '<>' of 'MW' en trafos is een 1x7 array met integers.
% Waarbij trafos(5)=0,1,2 en trafos(6)=0,2 in het geval dat vulling = '<>' of 'MW'.


% De periode van dubbelperiodieke patronen hebben we meermalen nodig.
dubbelper = gcd(m,n);
% Bijhouden welke patronen al geweest zijn.
gehad = logical(zeros(size(q)));
% een waitbar bijhouden
wachtmeter = waitbar(0,sprintf('Bezig van alle patronen de baan van equivalente patronen te bepalen\n Teller langs alle patronen gaat steeds sneller lopen...'));


% Maken van de voortbrengende operaties.
e = reshape([1:m*n],m,n);
% ----- x ------
% De voortbrenger x = x^trafos(1) y^trafos(2), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordex.
mod(trafos(1),m); x = [e(ans+1:end,:); e(1:ans,:)]; ordex = m/gcd(m,ans);
mod(trafos(2),n); Y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordeY = n/gcd(n,ans);
x = Y(x); ordex = lcm(ordex,ordeY); clear Y ordeY;
% ----- y -----
% De voortbrenger y = x^trafos(3) y^trafos(4), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordey.
mod(trafos(3),m); X = [e(ans+1:end,:); e(1:ans,:)]; ordeX = m/gcd(m,ans);
mod(trafos(4),n); y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordey = n/gcd(n,ans);
y = y(X); ordey = lcm(ordeX,ordey); clear X ordeX;
% ----- s -----
% De voortbrengende spiegeling, met orde ordes en teken sgns. (in de switch trafos(5) voor het geval vulling == '\/')
% Merk op dat ordes(1) = de orde van s voor niet-dubbelperiodieke patronen, terwijl ordes(2) het is voor dubbel-periodieke.
% Hiermee wordt geregeld dat s niet wordt toegepast op niet-dubbelperiodieke patronen als m~=n.
% N.B. Als m==n dan zijn alle patronen dubbelperiodiek (met periode gcd(m,n)==gcd(m,m)==gcd(n,n)==m==n).
switch mod(trafos(5),5)
case 0 % geen spiegelingen.
    s = e; ordes = [1 1]; sgns = 0;
case 1 % spiegeling in de verticale as.
    s = e(:,end:-1:1); ordes = [2 2]; sgns = 1;
case 2 % spiegeling in de horizontale as.
    s = e(end:-1:1,:); ordes = [2 2]; sgns = 1;
case 3 % spiegeling in de stijgende diagonaal.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    s = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = s(dubbelper*(i+1):-1:dubbelper*i+1,dubbelper*(j+1):-1:dubbelper*j+1)';
    end;end
    sgns = 0;
    % Tenslotte wordt ordes op 1 gezet voor niet-dubbelperiodieke patronen, zodat hij daarvoor niet meedoet.
    ordes = [1 2];
case 4 % spiegeling in de dalende diagonaal.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    s = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1))';
    end;end
    sgns = 0;
    % Tenslotte wordt ordes op 1 gezet voor niet-dubbelperiodieke patronen, zodat hij daarvoor niet meedoet.
    ordes = [1 2];
otherwise
    error('trafos(5) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if vulling == '-|' sgns = (ordes(2)-1)*mod(sgns+1,2); end % voor vulling == '-|' zijn de locale spiegeleigenschappen precies andersom.
if vulling == 'x+' sgns = 0; end         % voor vulling == 'x+' zorgt spiegelen nooit voor een omkering.
if vulling == 'sz' sgns = (ordes(2)-1); end % voor vulling == 'sz' zorgt spiegelen altijd voor een omkering.
if vulling == '<>' & mod(trafos(5),5) == 2 sgns = 0; end % Voor vulling = '<>' is alleen voor horizontale spiegeling het teken anders dan voor '\/'.
if vulling == 'MW' & mod(trafos(5),5) == 1 sgns = 0; end % Voor vulling = 'MW' is alleen voor verticale spiegeling het teken anders dan voor '\/'.
% ----- d -----
% De voortbrenger d = d^trafos(6), met d in rl draaiing kwartslag rechtsom. De orde is orded, het teken is sgnd.
% Merk op dat orded(1) = de orde van d voor niet-dubbelperiodieke patronen, terwijl orded(2) het is voor dubbel-periodieke.
% Net zo is d(:,:,1) de voortbrenger voor niet-dubbelperiodieke patronen, en d(:,:,2) die voor dubbelperiodiek.
% en sgnd(1) resp. sgnd(2) het bijbehorende teken.
% Hiermee wordt geregeld dat d als d^2 wordt toegepast op niet-dubbelperiodieke patronen als m~=n.
% N.B. Als m==n dan zijn alle patronen dubbelperiodiek (met periode gcd(m,n)==gcd(m,m)==gcd(n,n)==m==n).
switch mod(trafos(6),4)
case 0 % geen draaiingen.
    d(:,:,1) = e; d(:,:,2) = e; orded = [1 1]; sgnd = [0 0];
case 1 % draaiing kwartslag rechtsom.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    d = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            d(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = d(dubbelper*(i+1):-1:dubbelper*i+1,dubbelper*j+1:dubbelper*(j+1))';
    end;end
    % We zetten d op d^2 voor de niet-dubbelperiodieke patronen, en op d per deelvierkantje voor de dubbel-periodieke.
    d(:,:,2) = d; d(:,:,1) = e(end:-1:1,end:-1:1);
    % Met bijbehorend teken sgnd en orde orded.
    sgnd = [0 1]; orded = [2 4];
case 2 % draaiing kwartslag rechtsom.
    d = e(end:-1:1,end:-1:1); d(:,:,2) = d; orded = [2 2]; sgnd = [0 0];
case 3 % draaiing kwartslag linksom.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    d = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            d(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = d(dubbelper*i+1:dubbelper*(i+1),dubbelper*(j+1):-1:dubbelper*j+1)';
    end;end
    % We zetten d op e voor de niet-dubbelperiodieke patronen, en op d per deelvierkantje voor de dubbel-periodieke.
    d(:,:,2) = d; d(:,:,1) = e;
    % Met bijbehorend teken sgnd en orde orded.
    sgnd = [0 1]; orded = [1 4];
otherwise
    error('trafos(6) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if ismember(vulling, {'x+','sz'}) sgnd = [0 0]; end % voor vulling == 'x+' of 'sz' zorgt draaien nooit voor een omkering.
if ismember(vulling, {'<>','MW'}) sgnd = (orded-1) .* [1 1]; end % Bij vulling = '<>' of 'MW' zorgt puntspiegeling voor een omkering.
% ----- f -----
% Wel of niet de flip-transformatie
if mod(trafos(7),2) ordef = 2; else ordef = 1; end
 % Je zou ook sgnf=1 resp. 0 kunnen definieren, maar dat is niet nodig omdat je toch al weet dat f~=0 alleen als sgnf=1.


% loop over alle patronen
for r=1:prod(size(q))
    waitbar(r/prod(size(q)), wachtmeter);
if ~gehad(r) % Als we dit patroon nog niet hebben gehad, dan gaan we alle beelden maken (anders hoeven we deze niet meer te doen).
    % Het patroon horend bij r bepalen
    p = reshape(dec2bin(q(r),m*n),m,n);

    % Bepalen of het patroon dubbelperiodiek is (pisper=2) of niet (pisper=1)
    if m==n
        pisper = 2; % Als m==n hoeven we niets te checken, dan is het patroon altijd dubbelperiodiek.
    elseif p([e(dubbelper+1:end,end+1-dubbelper:end), e(dubbelper+1:end,1:end-dubbelper); e(1:dubbelper,end+1-dubbelper:end), e(1:dubbelper,1:end-dubbelper)])==p
        % Hier staat de eis dat p(x^dubbelper y^dubbelper)==p. Je kunt bewijzen dat dit equivalent is met:
		% p([e(dubbelper+1:end,:); e(1:dubbelper,:)])==p & p([e(:,end+1-dubbelper:end), e(:,1:end-dubbelper)])==p
        % Namelijk, (x^dubbelper y^dubbelper)^k = x^dubbelper voor zekere (unieke) k en (x^dubbelper y^dubbelper)^l = x^dubbelper voor zekere (unieke) l.
        pisper = 2;
    else
        pisper = 1;
    end

    % loop over alle groepselementen.
    xi = e; % initialisatie van de x-translatie waar we mee bezig zijn.
    for i=0:ordex-1
    yj = e; % initialisatie van de y-translatie waar we mee bezig zijn.
    for j=0:ordey-1
        % We lopen de xi(yj(draaispiegelingen)) af voor deze translatie
        sk = e;
        for k=0:ordes(pisper)-1
        dl = e; % initialisatie van de draaiing waar we mee bezig zijn.
        for l=0:orded(pisper)-1
            % Het getransformeerde patroon p(g) bepalen.
            pg = yj(xi);
            pg = sk(pg);
            pg = dl(pg);
            pg = p(pg);
            for f=0:ordef-1
                if mod(k*sgns+l*sgnd(pisper)+f,2)~=0 % het teken van de draaispiegeling s^k d^l en een eventuele flip
                                             % Merk op dat f=f*sgnf omdat f~=0 alleen als sgnf=1
                    % diagonaal-inverterende draaispiegeling, dan moeten de locale elementen nog worden gedraaid
                    pf = (pg=='0'); % nu is pf echter een logical array,
                    pf = int2str(pf); % dat moet nog even teruggezet worden.
                    pf = pf(pf~=' '); % de spaties eruit halen. Nu is pf overigens wel weer een vector geworden; pf(e) is de matrix.
                else
                    % niet-inverterende draaispiegeling: elementen blijven hetzelfde
                    pf = pg;
                end
                % Tenslotte het nieuwe patroon als getal zien en vergelijken met de patronen in q.
                % Als ze gevonden worden, dan noemen we ze gelijk en markeren ze als gehad.
                gelijke = (q==bin2dec(reshape(pf,1,m*n)));
                q(gelijke) = q(r); gehad(gelijke) = logical(1);
            end % flip-loop
        dl = dl(d(:,:,pisper));
        end % d-loop
        sk = sk(s);
        end % s-loop
    yj = yj(y);
    end
    xi = xi(x);
    end % translatie-loop
end % if patroon nog niet gehad
end % loop over patronen

% de waitbar weer weghalen
close(wachtmeter)