% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie bepaalt alle onafhankelijke patronen voor Robert Nottrot.
% De patronen worden gevormd door elementen in een rooster van m bij n vakjes, die twee waarden aan kunnen nemen.
% Vervolgens wordt dit rooster periodiek voortgezet over Z^2, oftewel, het is een patroon op Z_m x Z_n.
% Patronen worden hetzelfde genoemd als ze door rotatie, spiegeling of translatie in elkaar overgaan.
% Patronen worden gerepresenteerd door een getal. De binaire ontwikkeling ervan staat voor het patroon:
% 0 <-> '\','-','x','s','<' of 'M', 1 <-> '/','|','+','z','>' of 'W',
% gegeven een getal q, is p = dec2bin(q,m*n) de binaire ontwikkeling en p(e)=reshape(p,m,n) het bijbehorende patroon.
% De UITVOER p is een vector waarin elke p(r) een getal is waarvan de binaire ontwikkeling het patroon bepaalt.
% Deze routine geeft dezelfde uitvoer als p = double(unique(bepaalallebanen(m,n))),
% maar berekent dat op een slimmere manier die niet zo enorm veel geheugen kost,
% waarbij echter zowel de volgorde als de gekozen representanten anders kunnen zijn!
% Er wordt bovendien een factor 4 in rekentijd gewonnen.
%
% De INVOER-parameter vulling met mogelijke waarden 'x+', '-|', 'sz', '\/', '<>' of 'MW' bepaalt wat de symmetrie is
% van de elementen waaruit de patronen worden opgebouwd, oftewel wat het teken is van s en d.
% Namelijk:
%   - 'x+': 'x' en '+' blijven beide behouden bij spiegeling s en draaiing d, dus sgns = 0 = sgnd.
%   - '-|': '-' en '|' blijven behouden bij spiegeling s, maar gaan in elkaar over bij draaiing d, dus sgns=0, sgnd=1.
%   - 'sz': Hierbij staat 's' eigenlijk voor \-  /  en 'z' voor \  -/  Deze zijn beide symmetrisch onder draaiing d,
%                                             \ /|              |\ /   maar gaan in elkaar over onder spiegeling s.
%                                              X                  X    Dus sgns = 1, sgnd = 0.
%                                            |/ \                / \|
%                                            /  -\              /-  \
%   - '\/': Dit is de DEFAULT-waarde waar de oorspronkelijke vraag van Robert Nottrot over ging.
%           '\' en '/' gaan zowel onder spiegeling s als onder draaiing d in elkaar over, dus sgns = 1 = sgnd.
%   - '<>': Bij deze locale elementen mag draaiing d niet gebruikt worden, omdat je dan meer dan twee elementen
%           zou krijgen ('v' en '^' erbij) en dat vergt nog meer programmeer-veranderingen. Puntspiegeling d^2
%           voert deze elementen in elkaar over, net als spiegeling s, dus sgns = 1 = sgndd.
%   - 'MW': Ook hierbij mag draaiing d niet gebruikt worden, alleen puntspiegeling d^2, er geldt sgns = 0, sgndd = 1.
%               Deze laatste twee zullen overigens vaak hetzelfde resultaat geven!
%
% De INVOER-parameter trafos wordt gebruikt om eventueel bepaalde transformatie's uit de groep te weren, als volgt:
%   - trafos(1:2) bepaalt de voortbrenger voor in principe de verticale translatie's, zoals
%   - trafos(3:4) bepaalt wat de voortbrenger voor in principe de horizontale translatie's is:
%       [0 0] = deze translatie mag niet gebruikt worden.
%       [i j] = x^i y^j: i stappen naar boven (m-i naar beneden) en j naar rechs (n-j naar links) is de voortbrenger.
%   - trafos(5) bepaalt welke spiegeling als voortbrenger wordt gebruikt:
%       0 = geen spiegelingen
%       1 = spiegeling s, in verticale as
%       2 = spiegeling sd^2, in horizontale as
%       3 = spiegeling sd^3 in stijgende diagonaal (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%       4 = spiegeling sd in dalende diagonaal (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%     Merk op dat het geen zin heeft om twee verschillende spiegelingen als voortbrengers op te voeren,
%     omdat dat een draaiing is die met vulling(6) wordt bepaald. Verder maakt het niet uit welke van
%     de spiegelingen als voortbrenger wordt gebruikt als je ook alle draaiingen toelaat. Met dien verstande
%     dat het kiezen van sd^3 of sd voor m~=n ertoe leidt dat spiegelen alleen op dubbelperiodieke wordt toegepast.
%   - trafos(6) bepaalt welke draaiing als voortbrenger wordt gebruikt:
%       0 = draaiingen worden niet toegestaan
%       1 = draaiing kwartslag rechtsom (voor m~=n alleen op dubbelperiodieke, op andere d^2, puntspiegeling)
%       2 = draaiing d^2 over 180 graden, puntspiegeling (ook als m~=n dan geeft dit d^2, puntspiegeling)
%       3 = draaiing d^3 kwartslag linksom (voor m~=n alleen op dubbelperiodieke, op andere de identiteit)
%              LET OP: Voor m~=n geeft dit programma met voortbrenger d^3 dus andere resultaten dan met voortbrenger d,
%              ook al is dat niet wat je zou verwachten, omdat er nu anders op de niet-dubbelperiodieke wordt gewerkt!
%   - trafos(7) bepaalt of de flip-transformatie meedoet:
%       0 = de transformatie f doet niet mee
%       1 = de transformatie f mag gebruikt worden 
%           (deze verplaatst niets maar verandert elk element in zijn tegenhanger, is dus e met negatief teken).
% Om een en ander op een meer intuitieve manier in te voeren mag trafos ook een string zijn
% waarin het aantal keren dat x,Y,X,y,s,d en f voorkomen de getallen trafos(1:7) vervangen.
% (x,Y worden dus bij elkaar genomen tot trafos(1:2) en X,y worden samen genomen tot trafos(3:4)).
% Voor spiegelingen is dit wellicht enigszins tegen-intuitief: bijvoorbeeld sd^3 moet als 'sss' gecodeerd worden,
% terwijl 'sddd' geinterpreteerd zou worden als trafos(5:6) = [1 3].

function p = bepaalpatronen(m,n, vulling, trafos)

% checken dat m,n twee gehele getallen zijn.
if prod([size(m), size(n)]) ~= 1 | ceil([m,n]) ~= [m,n] | m <= 0 | n <=0
    error('m and n must be positive integers!');
end
% DEFAULT-waarde voor trafos en vulling
if nargin < 4
    trafos = [1 0 0 1 1 1 0];
if nargin < 3
    vulling = '\/';
end;end
if isempty(vulling) vulling = '\/'; end % vulling = '' is placeholder.

% vulling in een van de zes standaard-vormen brengen.
switch vulling
case {'\/','\','/','/\','\\','//'}
    vulling = '\/';
case {'-|','-','|','|-','--','||'}
    vulling = '-|';
case {'x+','x','+','+x','xx','++'}
    vulling = 'x+';
case {'sz','s','z','zs','ss','zz'}
    vulling = 'sz';
case {'<>','<','>','><','<<','>>'}
    vulling = '<>';
case {'MW','M','W','WM','MM','WW'}
    vulling = 'MW';
otherwise
    error(['vulling = ''' vulling ''' could not be interpreted as code for local symmetry!'])
end % switch vulling
% trafos in juiste vorm brengen
trafos = reshape(trafos,1,prod(size(trafos)));
if ischar(trafos)
    invoertrafos=trafos;
    trafos = zeros(1,5);
    trafos(1) = length(find(invoertrafos=='x'));
    trafos(2) = length(find(invoertrafos=='Y'));
    trafos(3) = length(find(invoertrafos=='X'));
    trafos(4) = length(find(invoertrafos=='y'));
    trafos(5) = length(find(invoertrafos=='s'));
    trafos(6) = length(find(invoertrafos=='d'));
    trafos(7) = length(find(invoertrafos=='f'));
    clear invoertrafos;
end % if trafos string -> getallen
if length(trafos) < 7 defaulttrafos = [1 0 0 1 1 1 0]; trafos = [trafos defaulttrafos(length(trafos)+1,end)]; end % trafos zo nodig aanvullen met DEFAULT-waarden.
trafos = floor(trafos(1:7));
if ismember(vulling(1:2), {'<>', 'MW'})
    if ismember(mod(trafos(5),5),[3 4]) trafos(5) = 0; end % Voor vulling = '<>' of 'MW' wordt geen diagonale spiegeling toegestaan.
	if mod(trafos(6),4) trafos(6) = 2; end % Voor vulling = '<>' of 'MW' wordt alleen puntspiegeling d^2 toegestaan.
end
% Vanaf nu is vulling = '\/', '-|', 'x+', 'sz', '<>' of 'MW' en trafos is een 1x7 array met integers.
% Waarbij trafos(5)=0,1,2 en trafos(6)=0,2 in het geval dat vulling = '<>' of 'MW'.


% De periode van dubbelperiodieke patronen hebben we meermalen nodig.
dubbelper = gcd(m,n);
% ten behoeve van de waitbar, bepalen we vast het totale aantal patronen dat gezocht wordt.
klaar = aantalpatronen(m,n, vulling, trafos); % dit gaat tenslotte redelijk rap.
wachtmeter = waitbar(0, sprintf('Zoeken naar de %i onafhankelijke patronen', klaar));
p = zeros(klaar,1); teller = 0; % lijst van representanten initialiseren.

% Maken van de voortbrengende operaties.
e = reshape([1:m*n],m,n);
% ----- x ------
% De voortbrenger x = x^trafos(1) y^trafos(2), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordex.
mod(trafos(1),m); x = [e(ans+1:end,:); e(1:ans,:)]; ordex = m/gcd(m,ans);
mod(trafos(2),n); Y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordeY = n/gcd(n,ans);
x = Y(x); ordex = lcm(ordex,ordeY); clear Y ordeY;
% ----- y -----
% De voortbrenger y = x^trafos(3) y^trafos(4), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordey.
mod(trafos(3),m); X = [e(ans+1:end,:); e(1:ans,:)]; ordeX = m/gcd(m,ans);
mod(trafos(4),n); y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordey = n/gcd(n,ans);
y = y(X); ordey = lcm(ordeX,ordey); clear X ordeX;
% ----- s -----
% De voortbrengende spiegeling, met orde ordes en teken sgns. (in de switch trafos(5) voor het geval vulling == '\/')
% Merk op dat ordes(1) = de orde van s voor niet-dubbelperiodieke patronen, terwijl ordes(2) het is voor dubbel-periodieke.
% Hiermee wordt geregeld dat s niet wordt toegepast op niet-dubbelperiodieke patronen als m~=n.
% N.B. Als m==n dan zijn alle patronen dubbelperiodiek (met periode gcd(m,n)==gcd(m,m)==gcd(n,n)==m==n).
switch mod(trafos(5),5)
case 0 % geen spiegelingen.
    s = e; ordes = [1 1]; sgns = 0;
case 1 % spiegeling in de verticale as.
    s = e(:,end:-1:1); ordes = [2 2]; sgns = 1;
case 2 % spiegeling in de horizontale as.
    s = e(end:-1:1,:); ordes = [2 2]; sgns = 1;
case 3 % spiegeling in de stijgende diagonaal.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    s = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = s(dubbelper*(i+1):-1:dubbelper*i+1,dubbelper*(j+1):-1:dubbelper*j+1)';
    end;end
    sgns = 0;
    % Tenslotte wordt ordes op 1 gezet voor niet-dubbelperiodieke patronen, zodat hij daarvoor niet meedoet.
    ordes = [1 2];
case 4 % spiegeling in de dalende diagonaal.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    s = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1))';
    end;end
    sgns = 0;
    % Tenslotte wordt ordes op 1 gezet voor niet-dubbelperiodieke patronen, zodat hij daarvoor niet meedoet.
    ordes = [1 2];
otherwise
    error('trafos(5) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if vulling == '-|' sgns = (ordes(2)-1)*mod(sgns+1,2); end % voor vulling == '-|' zijn de locale spiegeleigenschappen precies andersom.
if vulling == 'x+' sgns = 0; end         % voor vulling == 'x+' zorgt spiegelen nooit voor een omkering.
if vulling == 'sz' sgns = (ordes(2)-1); end % voor vulling == 'sz' zorgt spiegelen altijd voor een omkering.
if vulling == '<>' & mod(trafos(5),5) == 2 sgns = 0; end % Voor vulling = '<>' is alleen voor horizontale spiegeling het teken anders dan voor '\/'.
if vulling == 'MW' & mod(trafos(5),5) == 1 sgns = 0; end % Voor vulling = 'MW' is alleen voor verticale spiegeling het teken anders dan voor '\/'.
% ----- d -----
% De voortbrenger d = d^trafos(6), met d in rl draaiing kwartslag rechtsom. De orde is orded, het teken is sgnd.
% Merk op dat orded(1) = de orde van d voor niet-dubbelperiodieke patronen, terwijl orded(2) het is voor dubbel-periodieke.
% Net zo is d(:,:,1) de voortbrenger voor niet-dubbelperiodieke patronen, en d(:,:,2) die voor dubbelperiodiek.
% en sgnd(1) resp. sgnd(2) het bijbehorende teken.
% Hiermee wordt geregeld dat d als d^2 wordt toegepast op niet-dubbelperiodieke patronen als m~=n.
% N.B. Als m==n dan zijn alle patronen dubbelperiodiek (met periode gcd(m,n)==gcd(m,m)==gcd(n,n)==m==n).
switch mod(trafos(6),4)
case 0 % geen draaiingen.
    d(:,:,1) = e; d(:,:,2) = e; orded = [1 1]; sgnd = [0 0];
case 1 % draaiing kwartslag rechtsom.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    d = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            d(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = d(dubbelper*(i+1):-1:dubbelper*i+1,dubbelper*j+1:dubbelper*(j+1))';
    end;end
    % We zetten d op d^2 voor de niet-dubbelperiodieke patronen, en op d per deelvierkantje voor de dubbel-periodieke.
    d(:,:,2) = d; d(:,:,1) = e(end:-1:1,end:-1:1);
    % Met bijbehorend teken sgnd en orde orded.
    sgnd = [0 1]; orded = [2 4];
case 2 % draaiing kwartslag rechtsom.
    d = e(end:-1:1,end:-1:1); d(:,:,2) = d; orded = [2 2]; sgnd = [0 0];
case 3 % draaiing kwartslag linksom.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    d = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            d(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = d(dubbelper*i+1:dubbelper*(i+1),dubbelper*(j+1):-1:dubbelper*j+1)';
    end;end
    % We zetten d op e voor de niet-dubbelperiodieke patronen, en op d per deelvierkantje voor de dubbel-periodieke.
    d(:,:,2) = d; d(:,:,1) = e;
    % Met bijbehorend teken sgnd en orde orded.
    sgnd = [0 1]; orded = [1 4];
otherwise
    error('trafos(6) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if ismember(vulling, {'x+','sz'}) sgnd = [0 0]; end % voor vulling == 'x+' of 'sz' zorgt draaien nooit voor een omkering.
if ismember(vulling, {'<>','MW'}) sgnd = (orded-1) .* [1 1]; end % Bij vulling = '<>' of 'MW' zorgt puntspiegeling voor een omkering.

if sgnd(1) dd(:,:,1) = d(d(:,:,1)); else dd(:,:,1) = d(:,:,1); end
if sgnd(2) dd(:,:,2) = d(d(:,:,2)+m*n); else dd(:,:,2) = d(:,:,2); end % Als het teken van d negatief is,
                                    % dan moet hij 2 keer worden uitgevoerd om positief te blijven.
% ----- f -----
% Wel of niet de flip-transformatie
if mod(trafos(7),2) ordef = 2; else ordef = 1; end
 % Je zou ook sgnf=1 resp. 0 kunnen definieren, maar dat is niet nodig omdat je toch al weet dat f~=0 alleen als sgnf=1.
if sgns + sgnd(1) + ordef-1 noflip(1) = logical(0); else noflip(1) = logical(1); end
if sgns + sgnd(2) + ordef-1 noflip(2) = logical(0); else noflip(2) = logical(1); end % Als er geen flips gemaakt kunnen worden,
        % dan is het aantal nullen en enen in een baan constant, anders is er voor elke baan een representant
        % met maximaal zoveel enen als nullen.


% =================================================================================================================
% loop over het aantal enen in het patroon, elk patroon heeft een representant met maximaal zoveel enen als nullen,
% tenzij er geen negatieve draaispiegelingen zijn, maar dan is het nog simpeler:
% bij elk patroon met minder enen dan nullen is het geflipte patroon een ander onafhankelijk patroon.
% ---------------------------------------------------
% eerst doen we degenen met evenveel enen als nullen.
enen = m*n/2;
if enen == ceil(enen) % als m*n even is, dan heb je patronen met evenveel enen als nullen.
                           % daarvoor moet je ook de negatieve symmetrieen beschouwen.
    ind = nchoosek(1:m*n, enen); % ind(r,i) = de index-positie van de i^e een in het r^e patroon.
    q = char('0'*ones(nchoosek(m*n,enen),m*n)); % initialisatie van de binaire ontwikkelingen
    q([1:nchoosek(m*n,enen)]'*ones(1,enen) + nchoosek(m*n,enen)*(ind-1))='1'; % q(r,:) is de binaire representatie van het r^e patroon.
    q = bin2dec(q) ;clear ind % we maken meteen weer geheugen vrij.
                      % Ik vraag me echter af hoe zinvol dit is, want je hebt het als tussenstap wel nodig gehad.
                      % Voor de snelheid zou het beter zijn om de conversie pas later te maken,
                      % tenzij juist geheugenruimte de snelheid teveel beperkt. Je zou dit nog kunnen testen!

	% nu kunnen we een loop gaan maken over al deze patronen
	for r=1:nchoosek(m*n,enen)
	if q(r) ~= 0 % Als q(r)==0 dan zijn we dit patroon al tegengekomen en hoeven we deze dus niet meer te doen.
        % Het patroon toevoegen aan de lijst van representanten
        teller = teller + 1;
        p(teller) = q(r); waitbar(teller/klaar, wachtmeter); % je zou er i.v.m. snelheid voor kunnen kiezen om dit minder vaak te updaten, maar dat weegt waarschijnlijk toch niet op tegen de rest van de rekentijd.
        % Het patroon horend bij q(r) bepalen
        pr = reshape(dec2bin(q(r),m*n),m,n);

        % Bepalen of het patroon dubbelperiodiek is (pisper=2) of niet (pisper=1)
        if m==n
            pisper = 2; % Als m==n hoeven we niets te checken, dan is het patroon altijd dubbelperiodiek.
        elseif pr([e(dubbelper+1:end,end+1-dubbelper:end), e(dubbelper+1:end,1:end-dubbelper); e(1:dubbelper,end+1-dubbelper:end), e(1:dubbelper,1:end-dubbelper)])==pr
            % Hier staat de eis dat p(x^dubbelper y^dubbelper)==p. Je kunt bewijzen dat dit equivalent is met:
			% p([e(dubbelper+1:end,:); e(1:dubbelper,:)])==p & p([e(:,end+1-dubbelper:end), e(:,1:end-dubbelper)])==p
            % Namelijk, (x^dubbelper y^dubbelper)^k = x^dubbelper voor zekere (unieke) k en (x^dubbelper y^dubbelper)^l = x^dubbelper voor zekere (unieke) l.
            pisper = 2;
        else
            pisper = 1;
        end

        % loop over alle (positieve en negatieve) symmetrien.
        xi = e; % initialisatie van de x-translatie waar we mee bezig zijn.
        for i=0:ordex-1
        yj = e; % initialisatie van de y-translatie waar we mee bezig zijn.
        for j=0:ordey-1
            % We lopen de xi(yj(draaispiegelingen)) af voor deze translatie
            sk = e;
            for k=0:ordes(pisper)-1
            dl = e; % initialisatie van de draaiing waar we mee bezig zijn.
            for l=0:orded(pisper)-1
                % Het getransformeerde patroon pr(g) bepalen.
                pg = yj(xi);
                pg = sk(pg);
                pg = dl(pg);
                pg = pr(pg);
                for f=0:ordef-1
                    if mod(k*sgns+l*sgnd(pisper)+f,2)~=0 % het teken van de draaispiegeling s^k d^l en een eventuele flip
                                                 % Merk op dat f=f*sgnf omdat f~=0 alleen als sgnf=1
                        % diagonaal-inverterende draaispiegeling, dan moeten de locale elementen nog worden gedraaid
                        pf = (pg=='0'); % nu is pf echter een logical array,
                        pf = int2str(pf); % dat moet nog even teruggezet worden.
                        pf = pf(pf~=' '); % de spaties eruit halen. Nu is pf overigens wel weer een vector geworden; pf(e) is de matrix.
                    else
                        % niet-inverterende draaispiegeling: elementen blijven hetzelfde
                        pf = pg;
                    end
                    % Tenslotte het nieuwe patroon als getal zien en indiceren als gehad.
                    q(q == bin2dec(reshape(pf,1,m*n)))=0;
                end % flip-loop
            dl = dl(d(:,:,pisper));
            end % d-loop
            sk = sk(s);
            end % s-loop
        yj = yj(y);
        end
        xi = xi(x);
        end % translatie-loop
	end % if patroon nog niet gehad
	end % loop patronen met evenveel enen als nullen
end % if m*n even    
% nu loop over minder enen dan nullen    
for enen = ceil(m*n/2)-1:-1:0 % je begint met het zwaarste geval
    ind = nchoosek(0:m*n-1, enen); % ind(r,i) = de index-positie van de i^e een in het r^e patroon.
    q = char('0'*ones(nchoosek(m*n,enen),m*n)); % initialisatie van de binaire ontwikkelingen
    q([1:nchoosek(m*n,enen)]'*ones(1,enen) + nchoosek(m*n,enen)*ind)='1'; % q(r,:) is de binaire representatie van het r^e patroon.
    q = bin2dec(q) ;clear ind % we maken meteen weer geheugen vrij.
                      % Ik vraag me echter af hoe zinvol dit is, want je hebt het als tussenstap wel nodig gehad.
                      % Voor de snelheid zou het beter zijn om de conversie pas later te maken,
                      % tenzij juist geheugenruimte de snelheid teveel beperkt. Je zou dit nog kunnen testen!

	% nu kunnen we een loop gaan maken over al deze patronen
	for r=1:nchoosek(m*n,enen)
	if q(r) ~= -1 % Als q(r)==-1 dan zijn we dit patroon al tegegekomen en hoeven we deze dus niet meer te doen.
        % Het patroon toevoegen aan de lijst van representanten
        teller = teller + 1;
        p(teller) = q(r);

        % Het patroon horend bij q(r) bepalen
        pr = reshape(dec2bin(q(r),m*n),m,n);

        % Bepalen of het patroon dubbelperiodiek is (pisper=2) of niet (pisper=1)
        if m==n
            pisper = 2; % Als m==n hoeven we niets te checken, dan is het patroon altijd dubbelperiodiek.
        elseif pr([e(dubbelper+1:end,end+1-dubbelper:end), e(dubbelper+1:end,1:end-dubbelper); e(1:dubbelper,end+1-dubbelper:end), e(1:dubbelper,1:end-dubbelper)])==pr
            % Hier staat de eis dat p(x^dubbelper y^dubbelper)==p. Je kunt bewijzen dat dit equivalent is met:
			% p([e(dubbelper+1:end,:); e(1:dubbelper,:)])==p & p([e(:,end+1-dubbelper:end), e(:,1:end-dubbelper)])==p
            % Namelijk, (x^dubbelper y^dubbelper)^k = x^dubbelper voor zekere (unieke) k en (x^dubbelper y^dubbelper)^l = x^dubbelper voor zekere (unieke) l.
            pisper = 2;
        else
            pisper = 1;
        end

        if noflip(pisper) % Als er niet geflipt kan worden dan is het patroon met enen en nullen omgewisseld ook een nieuw onafhankelijk patroon.
            % We flippen nu dus het gevonden patroon.
            pg = (pr=='0'); % nu is pg echter een logical array,
            pg = int2str(pg); % dat moet nog even teruggezet worden.
            pg = pg(pg~=' '); % de spaties eruit halen. Nu is pg overigens wel weer een vector geworden; pg(e) is de matrix.
            teller = teller + 1;
            p(teller) = bin2dec(reshape(pg,1,m*n));
        end
        waitbar(teller/klaar, wachtmeter); % je zou er i.v.m. snelheid voor kunnen kiezen om dit minder vaak te updaten, maar dat weegt waarschijnlijk toch niet op tegen de rest van de rekentijd.

        % loop over alle positieve symmetrien.
            % (de negatieve kun je weglaten, want dan krijg je een representant met meer enen dan nullen)
        xi = e; % initialisatie van de x-translatie waar we mee bezig zijn.
        for i=0:ordex-1
        yj = e; % initialisatie van de y-translatie waar we mee bezig zijn.
        for j=0:ordey-1
            % Om slim door alleen de positieve draaispiegelingen te stappen is het sneller om met f te beginnen (is wel lastiger te programmeren).
			sk = e; dl = e; % om alleen de positieve draaispiegelingen te krijgen, moeten we f,dl en sk met elkaar in verband brengen.
			for f=0:1+(1-sgns)*(1-sgnd(pisper)):ordef-1
                % We lopen de xi(yj(draaispiegelingen)) af voor deze translatie
                for k=f*(1-sgnd(pisper)):1+sgns*(1-sgnd(pisper)):ordes(pisper)-1 % als sgnd==1 dan kan d corrigeren en maken we alle s,
                    % anders moet s voor correctie zorgen. Als die dat ook niet kan, dan had de f-loop hier nooit negatief binnen mogen komen.
                for l=mod(k*sgns+f,2):1+sgnd(pisper):orded(pisper)-1 % als sgnd = 0 dan kan d niet corrigeren voor negatief teken k*sgns+f*sgnf,
                    % dus mag die dan niet negatief binnenkomen; regel dat bij de s-loop (en f-loop).
                    % Merk overigens ook op dat f = f*sgnf, omdat f~=0 alleen als ordef=2 en sgnf=1.
                        % Het getransformeerde patroon pr(g) bepalen.
                        pg = yj(xi);
                        pg = sk(pg);
                        pg = dl(pg);
                        pg = pr(pg);
                        % We hebben het zo geregeld dat de draaispiegeling altijd positief is.
                        % Tenslotte het nieuwe patroon als getal zien en indiceren als gehad.
                        q(q == bin2dec(reshape(pg,1,m*n)))=-1;
                dl = dl(dd(:,:,pisper)); % om positief te blijven nemen we een stap van dd i.p.v. d.
                end % d-loop
                sk = s; if sgns if f dl = dl(dd(d(:,:,pisper)+(pisper-1)*m*n)); else dl = dl(d(:,:,pisper)); end;end % initialisatie van dl koppelen aan sk,
                    % als sgns==1, dan moet d extra corrigeren. Om de correspondentie dl = d^l te behouden,
                    % wordt bovendien een extra dd ingevoegd als f==1 (i.v.m. het verschoven begin).
                end % s-loop                
			if sgnd(pisper) sk = e; dl = d(:,:,pisper); else sk = s; dl = e; end % Als sgnd==1 dan mag nog gewoon met s=e begonnen worden, omdat dan d het nog wel positief maakt.
            end % flip-loop
        yj = yj(y);
        end
        xi = xi(x);
        end % translatie-loop
	end
	end % loop over patronen met dit aantal enen.
end % loop over enen

% de waitbar verwijderen
close(wachtmeter);