% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie checkt of een patroon op m bij n vakjes gegeven door
% (de binaire representatie van) het getal p gemaakt kan worden via
% draaispiegelingen en verschuivingen uit een van de patronen gegeven door de getallen q.
% In feite staat q dus als representant van zijn hele baan en check je of p in een van de banen zit.
% Voor de daadwerkelijke berekening wordt equivpatronen aangeroepen.
% Deze functie is een omhulsel dat een uitgebreide diagnostiek naar het scherm levert.
% Maar je kunt ook dezelfde uitvoer als van equivpatronen opvragen.
%
% INVOER-parameter uitbr maakt het mogelijk om een periodieke uitbreiding te geven, als in tekenpatroon.
% Verder bepalen vulling en trafos de transformatie-groep:
%  - vulling = waarmee getekend moet worden.
%           De eerste twee posities vulling(1:2) moeten zijn '\/', '-|', 'x+', 'sz', '<>' of 'MW' en bepalen
%           de aard van de symmetrie van de afzonderlijke vierkanten (zie uitleg bij aantalpatronen, bepaalpatronen, bepaalallebanen of bepaalbanen)
%           Indien aanwezig bepalen vulling(3:4) wat daadwerkelijk wordt ingevuld voor '0' resp. '1'
%               (bijvoorbeeld '\' resp. '/', '-' resp. '|', 'x' resp. '+', 's' resp. 'z', '<' resp. '>' of 'M' resp. 'W',
%                   maar kan van alles zijn, liefst met dezelfde locale symmetrie-eigenschappen)
%   - trafos = een 1x7-array van integers die bepaalt welke groepselementen mee mogen doen.
%       - trafos(1:2) en trafos(3:4) bepalen de voortbrengende translaties.
%       - trafos(5) bepaalt de voortbrengende spiegeling.
%       - trafos(6) bepaalt de voortbrengende draaiing.
%       - trafos(7) bepaalt of de 'flip-transformatie' mee mag doen.
%       Merk op dat ook een string waarvan de aantallen waarin x,Y,X,y,s,d en f voorkomen de elementen van de 1x7-array
%       bepalen. Zie voor een uitgebreidere omschrijving de uitleg bij aantalpatronen, bepaalpatronen, bepaalallebanen of bepaalbanen.
%
% N.B. een alternatieve methode voor hetzelfde probleem is ook geimplementeerd in equivpatronen met optie 'alt'.
% Om die aan te spreken kies je ook hier methode = 'alt', in plaats van de standaard methode = 'std'.
% uitbr = [k, l] bepaalt het aantal copies dat er van het fundamentaalgebied worden gemaakt.
% Ik zou eventueel ooit nog een optie mee kunnen geven om alle oplossingen te bepalen.

function [gevonden, baan, gx, gy, gs, gd, gf] = diagequivpatronen(p,q, m,n, uitbr, vulling, trafos, methode)

% De invoer in de geschikte vorm brengen.
afmetingen = [size(p), size(q)];
if length(afmetingen) > 4 error('Patterns p and q must be arrays of pattern numbers of dimension at most 2.'); end
p = reshape(p, prod(afmetingen(1:2)),1);
q = reshape(q, prod(afmetingen(3:4)),1);

% checken dat m,n twee gehele getallen zijn.
if prod([size(m), size(n)]) ~= 1 | ceil([m,n]) ~= [m,n] | m <= 0 | n <=0
    error('m and n must be positive integers!');
end

% DEFAULT-waarde voor methode en uitbr
if nargin < 8
    methode = 'std';
if nargin < 7
    trafos = [1 0 0 1 1 1 0];
if nargin < 6
    vulling = '\/\/';
if nargin < 5
    uitbr = [1,1];
end;end;end;end
if isempty(uitbr) % uitbr = [] is placeholder
    uitbr = [1,1];
end
if isempty(vulling) % vulling = '' is placeholder
    vulling = '\/\/';
end
if isempty(trafos) % trafos = [] is placeholder
    trafos = [1 0 0 1 1 1 0];
end

% checken dat uitbr van de goede vorm is
if prod(size(uitbr)) ~= 2 | ~(ceil(uitbr) == uitbr) | ~(uitbr > [0,0])
    error('Extension parameter uitbr is not of the form [k,l], with k,l positive integers!')
end
if uitbr(1)*m == 1 uitbr(1)=2; end % ik wil perse twee regels tot mijn beschikking hebben.

% checken dat vulling van de goede vorm is
vulling = reshape(vulling,1,prod(size(vulling)));
if length(vulling) <= 2
    switch vulling
    case {'\/','/\','\','/'}
        vulling = '\/\/';
    case {'-|','|-','-','|'}
        vulling = '-|-|';
    case {'x+','+x','x','+'}
        vulling = 'x+x+';
    case {'sz','zs','s','z'}
        vulling = 'szsz';
    case {'<>','><','<','>'}
        vulling = '<><>';
    case {'MW','WM','M','W'}
        vulling = 'MWMW';
    otherwise
        error(['Could not determine the requested symmetry from vulling = ' vulling '!'])
    end % switch vulling
else
    switch vulling(1:2)
    case {'\/','/\'}
        vulling(1:2) = '\/';
    case {'-|','|-'}
        vulling(1:2) = '-|';
    case {'x+','+x'}
        vulling(1:2) = 'x+';
    case {'sz','zs'}
        vulling(1:2) = 'sz';
    case {'<>','><'}
        vulling(1:2) = '<>';
    case {'MW','WM'}
        vulling(1:2) = 'MW';
    otherwise
        error(['Could not determine the requested symmetry from vulling(1:2) = ' vulling(1:2) '!'])
    end % switch vulling(1:2)
    if length(vulling)==3
        if vulling(3) ~= vulling(2) vulling(4)=vulling(2); else vulling(4)=vulling(1); end
    end
end % if length(vulling)<=2 of >2
% vanaf nu is vulling(1:2) '\/', '-|', 'x+', 'sz', '<>' of 'MW', vulling(3:4) de in te vullen karakters,

% checken dat trafos van de goede vorm is.
trafos = reshape(trafos,1,prod(size(trafos)));
if ischar(trafos)
    invoertrafos=trafos;
    trafos = zeros(1,7);
    trafos(1) = length(find(invoertrafos=='x'));
    trafos(2) = length(find(invoertrafos=='Y'));
    trafos(3) = length(find(invoertrafos=='X'));
    trafos(4) = length(find(invoertrafos=='y'));
    trafos(5) = length(find(invoertrafos=='s'));
    trafos(6) = length(find(invoertrafos=='d'));
    trafos(7) = length(find(invoertrafos=='f'));
    clear invoertrafos;
end % if trafos string -> getallen
if length(trafos) < 7 defaulttrafos = [1 0 0 1 1 1 0]; trafos = [trafos defaulttrafos(length(trafos)+1,end)]; end % trafos zo nodig aanvullen met DEFAULT-waarden.
trafos = floor(trafos(1:7));
if ismember(vulling(1:2), {'<>', 'MW'})
    if ismember(mod(trafos(5),5),[3 4]) trafos(5) = 0; end % Voor vulling = '<>' of 'MW' wordt geen diagonale spiegeling toegestaan.
	if mod(trafos(6),4) trafos(6) = 2; end % Voor vulling = '<>' of 'MW' wordt alleen puntspiegeling d^2 toegestaan.
end
% Vanaf nu is trafos een 1x7 array met integers.
% Waarbij trafos(5)=0,1,2 en trafos(6)=0,2 in het geval dat vulling = '<>' of 'MW'.


% De periode van dubbelperiodieke patronen hebben we meermalen nodig.
dubbelper = gcd(m,n);

% Maken van de voortbrengende operaties.
e = reshape([1:m*n],m,n);
% ----- x ------
% De voortbrenger x = x^trafos(1) y^trafos(2), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordex.
mod(trafos(1),m); x = [e(ans+1:end,:); e(1:ans,:)]; ordex = m/gcd(m,ans);
mod(trafos(2),n); Y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordeY = n/gcd(n,ans);
x = Y(x); ordex = lcm(ordex,ordeY); clear Y ordeY;
% ----- y -----
% De voortbrenger y = x^trafos(3) y^trafos(4), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordey.
mod(trafos(3),m); X = [e(ans+1:end,:); e(1:ans,:)]; ordeX = m/gcd(m,ans);
mod(trafos(4),n); y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordey = n/gcd(n,ans);
y = y(X); ordey = lcm(ordeX,ordey); clear X ordeX;
% ----- s -----
% De voortbrengende spiegeling, met orde ordes en teken sgns. (in de switch trafos(5) voor het geval vulling == '\/')
% Merk op dat ordes(1) = de orde van s voor niet-dubbelperiodieke patronen, terwijl ordes(2) het is voor dubbel-periodieke.
% Hiermee wordt geregeld dat s niet wordt toegepast op niet-dubbelperiodieke patronen als m~=n.
% N.B. Als m==n dan zijn alle patronen dubbelperiodiek (met periode gcd(m,n)==gcd(m,m)==gcd(n,n)==m==n).
switch mod(trafos(5),5)
case 0 % geen spiegelingen.
    s = e; ordes = [1 1]; sgns = 0; spiegel = '';
case 1 % spiegeling in de verticale as.
    s = e(:,end:-1:1); ordes = [2 2]; sgns = 1; spiegel = 's^%i';
case 2 % spiegeling in de horizontale as.
    s = e(end:-1:1,:); ordes = [2 2]; sgns = 1; spiegel = 'sdd^%i';
case 3 % spiegeling in de stijgende diagonaal.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    s = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = s(dubbelper*(i+1):-1:dubbelper*i+1,dubbelper*(j+1):-1:dubbelper*j+1)';
    end;end
    sgns = 0; spiegel = 'sddd^%i';
    % Tenslotte wordt ordes op 1 gezet voor niet-dubbelperiodieke patronen, zodat hij daarvoor niet meedoet.
    ordes = [1 2];
case 4 % spiegeling in de dalende diagonaal.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    s = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1))';
    end;end
    sgns = 0; spiegel = 'sd^%i';
    % Tenslotte wordt ordes op 1 gezet voor niet-dubbelperiodieke patronen, zodat hij daarvoor niet meedoet.
    ordes = [1 2];
otherwise
    error('trafos(5) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if vulling(1:2) == '-|' sgns = (ordes(2)-1)*mod(sgns+1,2); end % voor vulling == '-|' zijn de locale spiegeleigenschappen precies andersom.
if vulling(1:2) == 'x+' sgns = 0; end         % voor vulling == 'x+' zorgt spiegelen nooit voor een omkering.
if vulling(1:2) == 'sz' sgns = (ordes(2)-1); end % voor vulling == 'sz' zorgt spiegelen altijd voor een omkering.
if vulling(1:2) == '<>' & mod(trafos(5),5) == 2 sgns = 0; end % Voor vulling = '<>' is alleen voor horizontale spiegeling het teken anders dan voor '\/'.
if vulling(1:2) == 'MW' & mod(trafos(5),5) == 1 sgns = 0; end % Voor vulling = 'MW' is alleen voor verticale spiegeling het teken anders dan voor '\/'.
% ----- d -----
% De voortbrenger d = d^trafos(6), met d in rl draaiing kwartslag rechtsom. De orde is orded, het teken is sgnd.
% Merk op dat orded(1) = de orde van d voor niet-dubbelperiodieke patronen, terwijl orded(2) het is voor dubbel-periodieke.
% Net zo is d(:,:,1) de voortbrenger voor niet-dubbelperiodieke patronen, en d(:,:,2) die voor dubbelperiodiek.
% en sgnd(1) resp. sgnd(2) het bijbehorende teken.
% Hiermee wordt geregeld dat d als d^2 wordt toegepast op niet-dubbelperiodieke patronen als m~=n.
% N.B. Als m==n dan zijn alle patronen dubbelperiodiek (met periode gcd(m,n)==gcd(m,m)==gcd(n,n)==m==n).
switch mod(trafos(6),4)
case 0 % geen draaiingen.
    d(:,:,1) = e; d(:,:,2) = e; orded = [1 1]; sgnd = [0 0];
case 1 % draaiing kwartslag rechtsom.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    d = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            d(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = d(dubbelper*(i+1):-1:dubbelper*i+1,dubbelper*j+1:dubbelper*(j+1))';
    end;end
    % We zetten d op d^2 voor de niet-dubbelperiodieke patronen, en op d per deelvierkantje voor de dubbel-periodieke.
    d(:,:,2) = d; d(:,:,1) = e(end:-1:1,end:-1:1);
    % Met bijbehorend teken sgnd en orde orded.
    sgnd = [0 1]; orded = [2 4];
case 2 % draaiing kwartslag rechtsom.
    d = e(end:-1:1,end:-1:1); d(:,:,2) = d; orded = [2 2]; sgnd = [0 0];
case 3 % draaiing kwartslag linksom.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    d = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            d(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = d(dubbelper*i+1:dubbelper*(i+1),dubbelper*(j+1):-1:dubbelper*j+1)';
    end;end
    % We zetten d op e voor de niet-dubbelperiodieke patronen, en op d per deelvierkantje voor de dubbel-periodieke.
    d(:,:,2) = d; d(:,:,1) = e;
    % Met bijbehorend teken sgnd en orde orded.
    sgnd = [0 1]; orded = [1 4];
otherwise
    error('trafos(6) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if ismember(vulling(1:2), {'x+','sz'}) sgnd = [0 0]; end % voor vulling == 'x+' of 'sz' zorgt draaien nooit voor een omkering.
if ismember(vulling(1:2), {'<>','MW'}) sgnd = (orded-1) .* [1 1]; end % Bij vulling = '<>' of 'MW' zorgt puntspiegeling voor een omkering.
% ----- f -----
% Wel of niet de flip-transformatie
if mod(trafos(7),2) ordef = 2; else ordef = 1; end
 % Je zou ook sgnf=1 resp. 0 kunnen definieren, maar dat is niet nodig omdat je toch al weet dat f~=0 alleen als sgnf=1


% equivpatronen aanroepen om te bepalen welke patronen p in welke banen q zitten.
tic; [gevonden, baan, gx, gy, gs, gd, gf] = equivpatronen(p,q, m,n, vulling(1:2), trafos, methode); rekentijd = toc;
disp(sprintf('In %.3f s, zijn de volgende resultaten gevonden:', rekentijd));
disp(' ');

% De uiteindelijke patronen met de elementen in de rechthoeken tekenen.
% patronen p met de gevonden representant.
disp('De patronen p:             Representant q:            Transformatie:      Getransformeerde q:');
disp('                           (van zijn baan)               (q -> p)          (moet p zijn)');
disp(' ');
for i = 1:length(p)
    if gevonden(i)
        % Bepalen van de getransformeerde van q (bij wijze van check)
        qg = reshape(dec2bin(q(baan(i)),m*n),m,n);
        % Bepalen of het patroon dubbelperiodiek is (pisper=2) of niet (pisper=1)
        if m==n
            qisper = 2; % Als m==n hoeven we niets te checken, dan is het patroon altijd dubbelperiodiek.
        elseif qg([e(dubbelper+1:end,end+1-dubbelper:end), e(dubbelper+1:end,1:end-dubbelper); e(1:dubbelper,end+1-dubbelper:end), e(1:dubbelper,1:end-dubbelper)])==qg
            % Hier staat de eis dat q(x^dubbelper y^dubbelper)==q. Je kunt bewijzen dat dit equivalent is met:
			% q([e(dubbelper+1:end,:); e(1:dubbelper,:)])==q & q([e(:,end+1-dubbelper:end), e(:,1:end-dubbelper)])==q
            % Namelijk, (x^dubbelper y^dubbelper)^k = x^dubbelper voor zekere (unieke) k en (x^dubbelper y^dubbelper)^l = x^dubbelper voor zekere (unieke) l.
            qisper = 2;
        else
            qisper = 1;
        end
        xi=e; for r = 1:gx(i) xi = xi(x); end
        yj=e; for r = 1:gy(i) yj = yj(y); end
        sk=e; for r = 1:gs(i) sk = sk(s); end
        dl=e; for r = 1:gd(i)*orded(qisper)/orded(2) dl = dl(d(:,:,qisper)); end % gd(i) is weergegeven in termen van
            % de gevraagde voortbrenger d(:,:,2), maar voor niet-dubbelperiodieke patronen kan
            % alleen d(:,:,1) worden toegepast, maar dan moet je wel het aantal keren toepassen corrigeren.
        g = yj(xi); g = sk(g); g = dl(g); % de transformatie van de indices.
        qg = qg(g); %q transformeren.
        if mod(gs(i)*sgns+gd(i)*sgnd(2)+gf(i),2)~=0 % het teken van de draaispiegeling s^k d^l en een eventuele flip.
            % diagonaal-inverterende draaispiegeling, dan moeten de locale diagonalen nog worden gedraaid
            qg = (qg=='0'); % nu is pg echter een logical array,
            qg = int2str(qg); % dat moet nog even teruggezet worden.
            qg = qg(qg~=' '); % de spaties eruit halen. Nu is pg overigens wel weer een vector geworden; pg(e) is de matrix.
        end
        qg = bin2dec(reshape(qg,1,m*n)); % weer omzetten naar een getal.
        % Maken van de string die getoond moet worden.
        patronen = ''; patronen = [char(' '*ones(uitbr(1)*m,7)), tekenpatroon(p(i),m,n, uitbr, vulling(3:4)), ...
                                   char(' '*ones([m,-n].*uitbr+[0,27])), tekenpatroon(q(baan(i)),m,n, uitbr, vulling(3:4)),...
                                   char(' '*ones([m,-n].*uitbr+[0,47])), tekenpatroon(qg,m,n, uitbr, vulling(3:4))];
        patronen(1,1:6) = sprintf('%4i =', p(i)); % het getal van het patroon p toevoegen.
        patronen(1,28:33) = sprintf('%4i =', q(baan(i))); % het getal van de baan q toevoegen.
        patronen(1,75:80) = sprintf('%4i =', qg); % het getal van getransformeerde q, qg, toevoegen.
        % string-representatie van de groepstransformatie q -> p
        stringtrafo = sprintf('x^%iy^%i', mod(gx(i)*trafos(1)+gy(i)*trafos(3),m),mod(gx(i)*trafos(2)+gy(i)*trafos(4),n));
        stringtrafo = [stringtrafo, sprintf(spiegel, mod(gs(i),ordes(2)))]; % de voorgedefinieerde string spiegel is '', 's^%i','sdd^%i','sddd^%i' of 'sd^%i'.
        stringtrafo = [stringtrafo, sprintf('d^%if^%i', mod(gd(i)*trafos(6),4),mod(gf(i)*trafos(7),2)), '         '];
        % even checken of de string niet te lang is.
        if stringtrafo(16:end) == ' ' % vanaf 16 mag er geen informatie meer staan.
            stringtrafo = stringtrafo(1:15);
        else
            stringtrafo = ['[' num2str([gx(i),gy(i),gs(i),gd(i),gf(i)]) ']', '           '];
            if stringtrafo(16:end) == ' ' stringtrafo = stringtrafo(1:15); else stringtrafo = 'trafo past niet'; end
        end
        patronen(1,55:69) = stringtrafo;
        patronen(2,53:71) = [char('-'*ones(1,18)), '>']; % een pijl die aangeeft dat q tot qg getransformeerd wordt.
    else
        patronen = ''; patronen = [char(' '*ones(uitbr(1)*m,7)), tekenpatroon(p(i),m,n, uitbr, vulling(3:4)), ...
                                   char(' '*ones([m,-n].*uitbr+[0,20]))];
        patronen(1,1:6) = sprintf('%4i =', p(i)); % het getal van het patroon p toevoegen.
        patronen(1:2, 28:56) = ['Er is geen baan in q gevonden'; 'waar dit patroon p in zit.   '];
    end
    disp(patronen);
    disp(' ');
end
% alle representanten op een rijtje
disp('Alle representanten in q zijn:');disp(' ');
patronen = ''; i=0; for i = q' patronen = [patronen, tekenpatroon(i,m,n, uitbr, vulling(3:4)), char(' '*ones(uitbr(1)*m,5))]; end; disp(patronen)