% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie checkt of een patroon op m bij n vakjes gegeven door
% (de binaire representatie van) het getal p gemaakt kan worden via
% draaispiegelingen en verschuivingen uit een van de patronen gegeven door de getallen q.
% In feite staat q dus als representant van zijn hele baan en check je of p in een van de banen zit.
% Naast een logical true of false kan ook het groepselement g en de kleinste index ind
% gegeven worden waarvoor p = de trafo van q(ind), met trafo = x^gx y^gy s^gs d^gd f^gf
% (dus p = q(ind,d^gd(s^gs(y^gy(x^gx)))), omdat de trafo van de indices omgekeerd doorwerken, en de juiste omkering van de elementen).
% Als er meerdere representanten q van dezelfde baan of symmetrien in p zijn, dan is dit niet uniek
% en wordt de eerst gevonden oplossing gegeven. De uitvoer heeft dezelfde dimensies als p.
%
% Verder bepalen vulling en trafos de transformatie-groep:
%  - vulling = waarmee getekend moet worden.
%           De eerste twee posities vulling(1:2) moeten zijn '\/', '-|', 'x+', 'sz', '<>' of 'MW' en bepalen
%           de aard van de symmetrie van de afzonderlijke vierkanten (zie uitleg bij aantalpatronen, bepaalpatronen, bepaalallebanen of bepaalbanen)
%           Indien aanwezig bepalen vulling(3:4) wat daadwerkelijk wordt ingevuld voor '0' resp. '1'
%               (bijvoorbeeld '\' resp. '/', '-' resp. '|', 'x' resp. '+', 's' resp. 'z', '<' resp. '>' of 'M' resp. 'W',
%                   maar kan van alles zijn, liefst met dezelfde locale symmetrie-eigenschappen)
%   - trafos = een 1x7-array van integers die bepaalt welke groepselementen mee mogen doen.
%       - trafos(1:2) en trafos(3:4) bepalen de voortbrengende translaties.
%       - trafos(5) bepaalt de voortbrengende spiegeling.
%       - trafos(6) bepaalt de voortbrengende draaiing.
%       - trafos(7) bepaalt of de 'flip-transformatie' mee mag doen.
%       Merk op dat ook een string waarvan de aantallen waarin x,Y,X,y,s,d en f voorkomen de elementen van de 1x7-array
%       bepalen. Zie voor een uitgebreidere omschrijving de uitleg bij aantalpatronen, bepaalpatronen, bepaalallebanen of bepaalbanen.
%
% Ik zou eventueel ooit nog een optie mee kunnen geven om alle oplossingen te bepalen.
%
% N.B. Er zijn twee alternatieve methoden mogelijk:
% - 'std': De groepselementen loslaten op de banen q en telkens de p eruit filteren die daardoor bereikt worden.
% - 'alt': Een voor een alle patronen p langslopen en de bijbehorende baan q zoeken.
% De uitvoer is nagenoeg hetzelfde behalve dan dat via 'std' de trafo met de laagste
% waarden van gx,gy,gs,gd,gf worden gegeven, en via 'alt' de hoogste.

function [tf, qind, gx, gy, gs, gd, gf] = equivpatronen(p,q, m,n, vulling, trafos, methode)

% De invoer in de geschikte vorm brengen.
afmetingen = [size(p), size(q)];
if length(afmetingen) > 4 error('Patterns p and q must be arrays of pattern numbers of dimension at most 2.'); end
p = reshape(p, prod(afmetingen(1:2)),1);
q = reshape(q, prod(afmetingen(3:4)),1);

% checken dat m,n twee gehele getallen zijn.
if prod([size(m), size(n)]) ~= 1 | ceil([m,n]) ~= [m,n] | m <= 0 | n <=0
    error('m and n must be positive integers!');
end

% DEFAULT-waarde voor vulling, trafos en methode
if nargin < 7
    methode = 'std';
if nargin < 6
	trafos = [1 0 0 1 1 1 0];
if nargin < 5
    vulling = '\/';
end;end;end
if isempty(vulling) % vulling = '' is placeholder
    vulling = '\/';
end
if isempty(trafos) % trafos = [] is placeholder
    trafos = [1 0 0 1 1 1 0];
end

% vulling in een van de zes standaard-vormen brengen.
switch vulling
case {'\/','\','/','/\','\\','//'}
    vulling = '\/';
case {'-|','-','|','|-','--','||'}
    vulling = '-|';
case {'x+','x','+','+x','xx','++'}
    vulling = 'x+';
case {'sz','s','z','zs','ss','zz'}
    vulling = 'sz';
case {'<>','<','>','><','<<','>>'}
    vulling = '<>';
case {'MW','M','W','WM','MM','WW'}
    vulling = 'MW';
otherwise
    error(['vulling = ''' vulling ''' could not be interpreted as code for local symmetry!'])
end % switch vulling
% trafos in juiste vorm brengen
trafos = reshape(trafos,1,prod(size(trafos)));
if ischar(trafos)
    invoertrafos=trafos;
    trafos = zeros(1,7);
    trafos(1) = length(find(invoertrafos=='x'));
    trafos(2) = length(find(invoertrafos=='Y'));
    trafos(3) = length(find(invoertrafos=='X'));
    trafos(4) = length(find(invoertrafos=='y'));
    trafos(5) = length(find(invoertrafos=='s'));
    trafos(6) = length(find(invoertrafos=='d'));
    trafos(7) = length(find(invoertrafos=='f'));
    clear invoertrafos;
end % if trafos string -> getallen
if length(trafos) < 7 defaulttrafos = [1 0 0 1 1 1 0]; trafos = [trafos defaulttrafos(length(trafos)+1,end)]; end % trafos zo nodig aanvullen met DEFAULT-waarden.
trafos = floor(trafos(1:7));
if ismember(vulling(1:2), {'<>', 'MW'})
    if ismember(mod(trafos(5),5),[3 4]) trafos(5) = 0; end % Voor vulling = '<>' of 'MW' wordt geen diagonale spiegeling toegestaan.
	if mod(trafos(6),4) trafos(6) = 2; end % Voor vulling = '<>' of 'MW' wordt alleen puntspiegeling d^2 toegestaan.
end
% Vanaf nu is vulling = '\/', '-|', 'x+', 'sz', '<>' of 'MW' en trafos is een 1x7 array met integers.
% Waarbij trafos(5)=0,1,2 en trafos(6)=0,2 in het geval dat vulling = '<>' of 'MW'.


% De periode van dubbelperiodieke patronen hebben we meermalen nodig.
dubbelper = gcd(m,n);

% Maken van de voortbrengende operaties.
e = reshape([1:m*n],m,n);
% ----- x ------
% De voortbrenger x = x^trafos(1) y^trafos(2), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordex.
mod(trafos(1),m); x = [e(ans+1:end,:); e(1:ans,:)]; ordex = m/gcd(m,ans);
mod(trafos(2),n); Y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordeY = n/gcd(n,ans);
x = Y(x); ordex = lcm(ordex,ordeY); clear Y ordeY;
% ----- y -----
% De voortbrenger y = x^trafos(3) y^trafos(4), met x,y in rl de translatie van 1 (omhoog,naar rechts). De orde is ordey.
mod(trafos(3),m); X = [e(ans+1:end,:); e(1:ans,:)]; ordeX = m/gcd(m,ans);
mod(trafos(4),n); y = [e(:,end+1-ans:end), e(:,1:end-ans)]; ordey = n/gcd(n,ans);
y = y(X); ordey = lcm(ordeX,ordey); clear X ordeX;
% ----- s -----
% De voortbrengende spiegeling, met orde ordes en teken sgns. (in de switch trafos(5) voor het geval vulling == '\/')
% Merk op dat ordes(1) = de orde van s voor niet-dubbelperiodieke patronen, terwijl ordes(2) het is voor dubbel-periodieke.
% Hiermee wordt geregeld dat s niet wordt toegepast op niet-dubbelperiodieke patronen als m~=n.
% N.B. Als m==n dan zijn alle patronen dubbelperiodiek (met periode gcd(m,n)==gcd(m,m)==gcd(n,n)==m==n).
switch mod(trafos(5),5)
case 0 % geen spiegelingen.
    s = e; ordes = [1 1]; sgns = 0;
case 1 % spiegeling in de verticale as.
    s = e(:,end:-1:1); ordes = [2 2]; sgns = 1;
case 2 % spiegeling in de horizontale as.
    s = e(end:-1:1,:); ordes = [2 2]; sgns = 1;
case 3 % spiegeling in de stijgende diagonaal.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    s = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = s(dubbelper*(i+1):-1:dubbelper*i+1,dubbelper*(j+1):-1:dubbelper*j+1)';
    end;end
    sgns = 0;
    % Tenslotte wordt ordes op 1 gezet voor niet-dubbelperiodieke patronen, zodat hij daarvoor niet meedoet.
    ordes = [1 2];
case 4 % spiegeling in de dalende diagonaal.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    s = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = s(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1))';
    end;end
    sgns = 0;
    % Tenslotte wordt ordes op 1 gezet voor niet-dubbelperiodieke patronen, zodat hij daarvoor niet meedoet.
    ordes = [1 2];
otherwise
    error('trafos(5) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if vulling == '-|' sgns = (ordes(2)-1)*mod(sgns+1,2); end % voor vulling == '-|' zijn de locale spiegeleigenschappen precies andersom.
if vulling == 'x+' sgns = 0; end         % voor vulling == 'x+' zorgt spiegelen nooit voor een omkering.
if vulling == 'sz' sgns = (ordes(2)-1); end % voor vulling == 'sz' zorgt spiegelen altijd voor een omkering.
if vulling == '<>' & mod(trafos(5),5) == 2 sgns = 0; end % Voor vulling = '<>' is alleen voor horizontale spiegeling het teken anders dan voor '\/'.
if vulling == 'MW' & mod(trafos(5),5) == 1 sgns = 0; end % Voor vulling = 'MW' is alleen voor verticale spiegeling het teken anders dan voor '\/'.
% ----- d -----
% De voortbrenger d = d^trafos(6), met d in rl draaiing kwartslag rechtsom. De orde is orded, het teken is sgnd.
% Merk op dat orded(1) = de orde van d voor niet-dubbelperiodieke patronen, terwijl orded(2) het is voor dubbel-periodieke.
% Net zo is d(:,:,1) de voortbrenger voor niet-dubbelperiodieke patronen, en d(:,:,2) die voor dubbelperiodiek.
% en sgnd(1) resp. sgnd(2) het bijbehorende teken.
% Hiermee wordt geregeld dat d als d^2 wordt toegepast op niet-dubbelperiodieke patronen als m~=n.
% N.B. Als m==n dan zijn alle patronen dubbelperiodiek (met periode gcd(m,n)==gcd(m,m)==gcd(n,n)==m==n).
switch mod(trafos(6),4)
case 0 % geen draaiingen.
    d(:,:,1) = e; d(:,:,2) = e; orded = [1 1]; sgnd = [0 0];
case 1 % draaiing kwartslag rechtsom.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    d = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            d(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = d(dubbelper*(i+1):-1:dubbelper*i+1,dubbelper*j+1:dubbelper*(j+1))';
    end;end
    % We zetten d op d^2 voor de niet-dubbelperiodieke patronen, en op d per deelvierkantje voor de dubbel-periodieke.
    d(:,:,2) = d; d(:,:,1) = e(end:-1:1,end:-1:1);
    % Met bijbehorend teken sgnd en orde orded.
    sgnd = [0 1]; orded = [2 4];
case 2 % draaiing kwartslag rechtsom.
    d = e(end:-1:1,end:-1:1); d(:,:,2) = d; orded = [2 2]; sgnd = [0 0];
case 3 % draaiing kwartslag linksom.
    % loop over alle dubbelper x dubbelper deelvierkantjes; voor m==n is dit meteen het hele vierkant.
    d = e; for i = 0:m/dubbelper-1 for j = 0:n/dubbelper-1
            d(dubbelper*i+1:dubbelper*(i+1),dubbelper*j+1:dubbelper*(j+1)) = d(dubbelper*i+1:dubbelper*(i+1),dubbelper*(j+1):-1:dubbelper*j+1)';
    end;end
    % We zetten d op e voor de niet-dubbelperiodieke patronen, en op d per deelvierkantje voor de dubbel-periodieke.
    d(:,:,2) = d; d(:,:,1) = e;
    % Met bijbehorend teken sgnd en orde orded.
    sgnd = [0 1]; orded = [1 4];
otherwise
    error('trafos(6) is geen integer (zou niet moeten kunnen)!')
end
% Correctie van het teken voor andere gevallen.
if ismember(vulling, {'x+','sz'}) sgnd = [0 0]; end % voor vulling == 'x+' of 'sz' zorgt draaien nooit voor een omkering.
if ismember(vulling, {'<>','MW'}) sgnd = (orded-1) .* [1 1]; end % Bij vulling = '<>' of 'MW' zorgt puntspiegeling voor een omkering.
% ----- f -----
% Wel of niet de flip-transformatie
if mod(trafos(7),2) ordef = 2; else ordef = 1; end
 % Je zou ook sgnf=1 resp. 0 kunnen definieren, maar dat is niet nodig omdat je toch al weet dat f~=0 alleen als sgnf=1


% declaratie van de uitvoer die gevuld gaat worden
gx = ones(length(p),1)*-ordex;
gy = ones(length(p),1)*-ordey;
gs = ones(length(p),1)*-ordes(2);
gd = ones(length(p),1)*-orded(2);
gf = ones(length(p),1)*-ordef;
qind = zeros(length(p),1);
tf = logical(qind);

if length(q)*length(p) ~= 0 % als p of q leeg zijn, dan zul je natuurlijk niets vinden (maar wel foutmeldingen krijgen)
% Kiezen welke methode gebruikt wordt
switch methode
case 'std' % Methode: voor alle banen q tegelijk lopen over de groepselementen en de patronen p eruit filteren:
	% De representanten omzetten in binaire patronen.
	q = dec2bin(q,m*n);
	% deze houdt bij welke patronen nog niet gevonden zijn.
	teller = 1:length(p);

    % Van de dubbelperiodieke patronen voegen we de met de "verboden spiegelinge/draaiingen" getransformeerde patronen toe aan q.
    if m==n
        qisper = 2; % Als m==n hoeven we niets te checken, dan is het patroon altijd dubbelperiodiek.
            % En in dat geval voegen we niets toe maar, gaan we gewoon alles volgens dubbelperiodiek transformeren.
    elseif mod(trafos(5),5) <= 2 & mod(trafos(6)+1,2) % als mod(trafos(5),5)==0,1,2 en mod(trafos(6),2)==0,2
            % Dan hoeven we ook nooit onderscheid te maken tussen dubbelperiodiek of niet.
        qisper = 2;
    else
        % Voor alle patronen in q cheken of ze dubbelperiodiek zijn.
        qisper = ones(prod(afmetingen(3:4),1));
        for r = 1:prod(afmetingen(3:4))
            qr = reshape(q(r,:),m,n);
            if qr([e(dubbelper+1:end,end+1-dubbelper:end), e(dubbelper+1:end,1:end-dubbelper); e(1:dubbelper,end+1-dubbelper:end), e(1:dubbelper,1:end-dubbelper)])==qr
                % Hier staat de eis dat p(x^dubbelper y^dubbelper)==p. Je kunt bewijzen dat dit equivalent is met:
				% p([e(dubbelper+1:end,:); e(1:dubbelper,:)])==p & p([e(:,end+1-dubbelper:end), e(:,1:end-dubbelper)])==p
                % Namelijk, (x^dubbelper y^dubbelper)^k = x^dubbelper voor zekere (unieke) k en (x^dubbelper y^dubbelper)^l = x^dubbelper voor zekere (unieke) l.
                qisper(r) = 2;
            else
                qisper(r) = 1;
            end % if dubbelperiodiek.
        end % q-loop
        if min(qisper) == 2
            qisper = 2; % Als alles dubbelperiodiek is, dan doen we niet moeilijk, maar transformeren alles volgens dubbelperiodiek.
        else
            % Als een deel niet-dubbelperiodiek is, dan gaan we van de dubbelperiodieke patronen de "verboden" getransformeerden toevoegen.
            extra = (1+(mod(trafos(5),5)>=3))*(1+mod(trafos(6),2)*mod(trafos(6),4))-1; % zoveel kopien komen er van elk dubbelperiodieke patroon.
            afkomst = find(qisper==2); % onthouden van welke patronen kopien zijn gemaakt.
            q = [char(' '*ones(extra*length(afkomst),m*n)); q]; % De geheugenruimte voor de uitbreiding van q vast reserveren.
            for r = 1:length(afkomst)
                origineel = length(afkomst)*extra + afkomst(r); dl = d(:,:,2); % d initialiseren.
                for l = 1:mod(trafos(6),2)*mod(trafos(6),4) % De benodigde d-kopien
                    q((r-1)*extra+l,:) = reshape(q(origineel,dl),1,m*n);
					% Nog even flippen als het teken van dl negatief is:
                    if mod(l*sgnd(2),2)
                        qg = int2str(q((r-1)*extra+l,:)=='0');
                        q((r-1)*extra+l,:) = reshape(qg(qg~=' '),1,m*n);
                    end
                    origineel = [origineel (r-1)*extra+l]; % zorgen dat ook hier een s-kopie van komt.
                    dl = dl(d(:,:,2));
                end % d-loop
                if mod(trafos(5),5) >= 3 % Als we gespiegelde toe moeten voegen
                    q((r-1)*extra+l+[1:length(origineel)],:) = reshape(q(origineel,s(:,:,2)),length(origineel),m*n);
					% Nog even flippen als het teken van s negatief is:
                    if sgns
                        qg = int2str(q((r-1)*extra+l+[1:length(origineel)],:)=='0');
                        q((r-1)*extra+l+[1:length(origineel)],:) = reshape(qg(qg~=' '),length(origineel),m*n);
                    end
                end % s-if
            end % loop over dubbelperiodieke patronen
            qisper = 1; % Nu gaat alles slechts volgens niet-dubbelperiodiek getransformeerd worden.
            % En er zijn kopien van de dubbelperiodieke patronen q(afkomst) neergezet
            % in de eerste extra*length(afkomst) rijen van de array q,
            % in de volgorde d,dd,ddd,s,sd,sdd,sddd, voor zover van toepassing is.
        end % if alles dubbelperiodiek.
    end % if m==n of geen onderscheid
    % nieuwe afmetingen van q onthouden
    afmetingen(3:4) = size(q); afmetingen(4) = afmetingen(4)/m/n;

	% loop over alle groepselementen.
	i=0;xi = e; % initialisatie van de x-translatie waar we mee bezig zijn.
	while length(teller) > 0 & i < ordex % zolang er nog patronen p niet zijn gevonden gaan we door tot het laatste groepselement
	j=0;yj = e; % initialisatie van de y-translatie waar we mee bezig zijn.
	while length(teller) > 0 & j < ordey
        % We lopen de xi(yj(draaispiegelingen)) af voor deze translatie
        k=0;sk = e; % initialisatie van de spiegeling waar we mee bezig zijn.
        while length(teller) > 0 & k < ordes(qisper)
        l=0;dl = e; % initialisatie van de draaiing waar we mee bezig zijn.
        while length(teller) > 0 & l < orded(qisper)
            % De getransformeerde patronen q(g) bepalen.
            qg = yj(xi);
            qg = sk(qg);
            qg = dl(qg);
            qg = q(:,qg);
            f=0; % initialisatie van de flip
            while length(teller) > 0 & f < ordef
                if mod(k*sgns+l*sgnd(qisper)+f,2)~=0 % het teken van de draaispiegeling s^k d^l en een eventuele flip
                                             % Merk op dat f=f*sgnf omdat f~=0 alleen als sgnf=1
                    % diagonaal-inverterende draaispiegeling, dan moeten de locale diagonalen nog worden gedraaid
                    qf = (qg=='0'); % nu is qf echter een logical array,
                    qf = int2str(qf); % dat moet nog even teruggezet worden.
                    qf = qf(qf~=' '); % de spaties eruit halen. Nu is qf overigens wel weer een vector geworden;
                    qf = reshape(qf,prod(afmetingen(3:4)),m*n); % dus wordt nu de vorm hersteld.
                else
                    % niet-inverterende draaispiegeling: locale elementen blijven hetzelfde.
                    qf = qg;
                end
                % Tenslotte gaan we deze nieuwe patronen als getal zien en kijken of we patronen p hebben gemaakt
                [erin, welke] = ismember(p,bin2dec(qf));
                % teller(erin) zijn de indices van de gevonden patronen p.
                tf(teller(erin)) = logical(1);
                qind(teller(erin)) = welke(erin);
                gx(teller(erin)) = i;
                gy(teller(erin)) = j;
                gs(teller(erin)) = k;
                gd(teller(erin)) = l;
                gf(teller(erin)) = f;
                % de patronen die we hebben gehad worden geschrapt.
                p(erin)=[];
                teller(erin) = []; % doordat we met teller hetzelfde doen, blijven we weten bij welke oorspronkelijke index het hoort.
            f=f+1;
            end % flip-while
        l=l+1;dl = dl(d(:,:,qisper));
        end % d-while
        k=k+1;sk = sk(s);
        end % s-while
	j=j+1;yj = yj(y);
	end
	i=i+1;xi = xi(x);
	end % loop groepselementen

    % Nog even corrigeren als er naar kopieen van de dubbelperiodieke patronen wordt verwezen.
    if qisper == 1
        % Eerst de indices corrigeren voor het feit dat er een aantal kopieen voor staan.
        qind = qind - extra*length(afkomst);
        % Verder corrigeren voor het feit dat gd gegeven is in aantal keren d(:,:,1) i.p.v. d(:,:,2)
        gd = gd*orded(2)/2; % Bedenk dat d(:,:,1) = d^2 altijd als qisper==1 (of orded(2)==orded(1)==1, maar dan gd==0 altijd).
        for p = find(qind<=0)'
        if tf(p) % loop over de patronen die gevonden zijn voor een kopie van dubbelperiodieke baan.
            kopieind = mod(qind(p)-1,extra)+1;
            if kopieind > mod(trafos(6),2)*mod(trafos(6),4)
                % Als de gevonden kopie niet een van de d-kopieen is,
                % dan is het een s-kopie en moet d^gd nog met s gecommuteerd worden.
                gs(p) = 1; % Bedenk dat altijd ordes(qisper)=1 voor qisper==1, dus k==0 altijd en mod(gs(p)+1,2)==1 altijd.
                gd(p) = mod(3*gd(p) + kopieind-mod(trafos(6),2)*mod(trafos(6),4)-1, orded(2)); % dit is nu het aantal keren d(:,:,2).
			else
                % Nu zitten we kennelijk in de situatie zonder "verboden spiegeling" erin, dus is omrekenen simpel.
                % Merk op dat dit alleen wordt aangesproken wanneer mod(trafos(6),4)==1,3 en dus orded(2)==4.
                % gd is al omgerekend naar d(:,:,2).
                gd(p) = mod(gd(p) + kopieind, orded(2));
            end
            % qind laten verwijzen naar het oorspronkelijke (dubbelperiodieke) patroon
            qind(p) = afkomst(length(afkomst)-ceil(qind(p)/extra));
        else
            % Als het patroon niet gevonden was, dan willen we qind op 0 houden.
            qind(p) = 0;
        end;end % loop over gevonden patronen in banen van dubbelperiodieke kopieen
    end % if kopien gemaakt
case 'alt' % Methode: per patroon p de inverse transformaties uitvoeren en kijken of je in een baan q zit:
    % loop over alle patroongetallen p: we zoeken een baan waar p in zit.
	for teller=1:length(p)
        ind = [];
        % Het patroon horend bij p bepalen
        pat = reshape(dec2bin(p(teller),m*n),m,n);

        % Bepalen of het patroon dubbelperiodiek is (pisper=2) of niet (pisper=1)
        if m==n
            pisper = 2; % Als m==n hoeven we niets te checken, dan is het patroon altijd dubbelperiodiek.
        elseif pat([e(dubbelper+1:end,end+1-dubbelper:end), e(dubbelper+1:end,1:end-dubbelper); e(1:dubbelper,end+1-dubbelper:end), e(1:dubbelper,1:end-dubbelper)])==pat
            % Hier staat de eis dat p(x^dubbelper y^dubbelper)==p. Je kunt bewijzen dat dit equivalent is met:
			% p([e(dubbelper+1:end,:); e(1:dubbelper,:)])==p & p([e(:,end+1-dubbelper:end), e(:,1:end-dubbelper)])==p
            % Namelijk, (x^dubbelper y^dubbelper)^k = x^dubbelper voor zekere (unieke) k en (x^dubbelper y^dubbelper)^l = x^dubbelper voor zekere (unieke) l.
            pisper = 2;
        else
            pisper = 1;
        end

        % loop over alle groepselementen. (in tegenstelling tot anders is xi de inverse x^-i en ditto voor yj,sk,dl)
        i=ordex+1;xi = e; % initialisatie van de x-translatie waar we mee bezig zijn.
        while isempty(ind) & i > 1 % zolang er geen oplossing is gevonden gaan we door tot het laatste groepselement
        i=i-1;
        j=ordey+1;yj = e; % initialisatie van de y-translatie waar we mee bezig zijn.
        while isempty(ind) & j > 1
        j=j-1;
            % We lopen de xi(yj(draaispiegelingen)) af voor deze translatie
            k=ordes(pisper)+1;sk = e; % initialisatie van de spiegeling waar we mee bezig zijn.
            while isempty(ind) & k > 1
            k=k-1;
            l=orded(pisper)+1;dl = e; % initialisatie van de draaiing waar we mee bezig zijn.
            while isempty(ind) & l > 1
            l=l-1;
                % Het getransformeerde patroon p(g^(-1)) bepalen.
                % dit is in de omgekeerde volgorde als normaal, zodat het makkelijk is om te rekenen tot p=q(g).
                pg = sk(dl);
                pg = yj(pg);
                pg = xi(pg);
                pg = pat(pg);
                f=-1; % We doen f omgekeerd (gelukkig is f=f^-1 en commuteert met alles)
                while isempty(ind) & f < ordef-1 % Omdat we anders sgnf ook moeten gaan gebruiken.
                f=f+1; % Nu blijft f=0 en f=1 de juiste betekenis houden en is sgnf niet nodig.
                    if mod(k*sgns+l*sgnd(pisper)+f,2)~=0 % het teken van de draaispiegeling s^-k d^-l en een eventuele flip
                                                 % Merk op dat f=f*sgnf omdat f~=0 alleen als sgnf=1
                        % diagonaal-inverterende draaispiegeling, dan moeten de locale diagonalen nog worden gedraaid
                        pf = (pf=='0'); % nu is pf echter een logical array,
                        pf = int2str(pf); % dat moet nog even teruggezet worden.
                        pf = pf(pf~=' '); % de spaties eruit halen. Nu is pf overigens wel weer een vector geworden; pf(e) is de matrix.
                    else
                        % niet-inverterende draaipiegeling: elementen blijven hetzelfde
                        pf = pg;
                    end
                    % Tenslotte dit nieuwe patroon als getal zien en vergelijken met q
                    ind = find(q==bin2dec(reshape(pf,1,m*n)));
                end % flip-while
            dl = dl(d(:,:,pisper));
            end % d-while
            sk = sk(s);
            end % s-while
        yj = yj(y);
        end
        xi = xi(x);
        end % loop groepselementen

        % nu hebben we een oplossing gevonden (ind niet leeg), of p is onafhankelijk van alle patronen in q.
        tf(teller) = ~isempty(ind);
        if ~isempty(ind)
            qind(teller) = ind(end);
            gx(teller) = mod(i,ordex);
            gy(teller) = mod(j,ordey);
            gs(teller) = mod(k,ordes(pisper));
            gd(teller) = mod(l,orded(pisper)); % dit is nu in termen van d(:,:,pisper)
            gd(teller) = gd(teller)*orded(2)/orded(pisper); % en dit is in termen van d(:,:,2), zoals gevraagd wordt.
            gf(teller) = mod(f,ordef);
        end
	end % patronen p
otherwise
    error(['Er is geen alternatieve werkwijze methode = ' methode ' bekend!'])
end % switch methode
end % if length(p;q)

% Tenslotte de uitvoer nog in dezelfde shape brengen als p was.
tf = reshape(tf,afmetingen(1:2));
qind = reshape(qind,afmetingen(1:2));
gx = reshape(gx,afmetingen(1:2));
gy = reshape(gy,afmetingen(1:2));
gs = reshape(gs,afmetingen(1:2));
gd = reshape(gd,afmetingen(1:2));
gf = reshape(gf,afmetingen(1:2));