% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie bepaalt met bepaalpatronen alle onafhankelijke patronen voor Robert Nottrot.
% De patronen worden gevormd door elementen in een rooster van m bij n vakjes, die twee waarden aan kunnen nemen.
% Vervolgens wordt dit rooster periodiek voortgezet over Z^2, oftewel, het is een patroon op Z_m x Z_n.
% Patronen worden hetzelfde genoemd als ze door rotatie, spiegeling of translatie in elkaar overgaan.
%
% De taak van deze functie is het op overzichtelijke wijze uitvoeren naar scherm of bestand
% van alle zo gevonden onafhankelijke patronen.
%
% Invoer:
%  - m,n = de afmetingen van het fundamentaalgebied.
%  - uitbr = het aantal kopien van het fundamenteelgebied die je in het overzicht wilt krijgen.
%           Als uitbr == 0, dan worden er geen extensies toegevoegd in het overzicht.
%  - vulling = waarmee getekend moet worden.
%           De eerste twee posities vulling(1:2) moeten zijn '\/', '-|', 'x+', 'sz', '<>' of 'MW' en bepalen
%           de aard van de symmetrie van de afzonderlijke vierkanten (zie uitleg bij aantalpatronen, bepaalpatronen, bepaalallebanen of bepaalbanen)
%           Indien aanwezig bepalen vulling(3:4) wat daadwerkelijk wordt ingevuld voor '0' resp. '1'
%               (bijvoorbeeld '\' resp. '/', '-' resp. '|', 'x' resp. '+', 's' resp. 'z', '<' resp. '>' of 'M' resp. 'W',
%                   maar kan van alles zijn, liefst met dezelfde locale symmetrie-eigenschappen)
%   - trafos = een 1x7-array van integers die bepaalt welke groepselementen mee mogen doen.
%       - trafos(1:2) en trafos(3:4) bepalen de voortbrengende translaties.
%       - trafos(5) bepaalt de voortbrengende spiegeling.
%       - trafos(6) bepaalt de voortbrengende draaiing.
%       - trafos(7) bepaalt of de 'flip-transformatie' mee mag doen.
%       Merk op dat ook een string waarvan de aantallen waarin x,Y,X,y,s,d en f voorkomen de elementen van de 1x7-array
%       bepalen. Zie voor een uitgebreidere omschrijving de uitleg bij aantalpatronen, bepaalpatronen, bepaalallebanen of bepaalbanen.
%   - uitvoer = de naam van een bestand waar de uitvoer naar toe moet.
%           Indien uitvoer begint met 'scherm:' dan wordt zowel naar het scherm als naar het bestand
%           met de naam die volgt op 'scherm:' volgt, uitgevoerd, anders alleen naar het bestand.
%       Zo kun je ook het programma ook forceren zonder interactie te werken door vooraf te melden
%       wat er moet gebeuren als het bestand reeds bestaat. Ook wordt dan automatisch geannuleerd (met een warning)
%       als het bestand niet geopend kan worden. De code hiervoor is:
%           - 'erover:' als het bestand overschreven moet worden.
%           - 'append:' als er achter het bestand bijgeschreven moet worden.
%           - 'cancel:' als er dan niet gerekend hoeft te worden (handig als je het bestand alleen wilt maken als dat nodig is).

function schrijfpatronen(m,n,uitbr, vulling, trafos, uitvoer)

% DEFAULT-waarde voor uitvoer, vulling, trafos en uitbr
if nargin < 6
    uitvoer = 'scherm:';
if nargin < 5
    trafos = [1 0 0 1 1 1 0];
if nargin < 4
    vulling = '\/\/';
if nargin < 3
    uitbr = [0,0];
end;end;end;end
if isempty(uitbr) % uitbr = [] is placeholder
    uitbr = [0,0];
end
if isempty(vulling) % vulling = '' is placeholder
    vulling = '\/\/';
end
if isempty(trafos) % trafos = [] is placeholder
    trafos = [1 0 0 1 1 1 0];
end

% VORM VAN DE INVOER CHECKEN
% checken dat m,n twee gehele getallen zijn.
if prod([size(m), size(n)]) ~= 1 | ceil([m,n]) ~= [m,n] | m <= 0 | n <=0
    error('m and n must be positive integers!');
end

% checken dat uitbr van de goede vorm is
if prod(size(uitbr)) ~= 2 | ~(ceil(uitbr) == uitbr) | ~(uitbr > [-1,-1])
    error('Extension parameter uitbr is not of the form [k,l], with k,l positive integers!')
end

% checken dat vulling van de goede vorm is
vulling = reshape(vulling,1,prod(size(vulling)));
if length(vulling) <= 2
    switch vulling
    case {'\/','/\','\','/'}
        vulling = '\/\/';
    case {'-|','|-','-','|'}
        vulling = '-|-|';
    case {'x+','+x','x','+'}
        vulling = 'x+x+';
    case {'sz','zs','s','z'}
        vulling = 'szsz';
    case {'<>','><','<','>'}
        vulling = '<><>';
    case {'MW','WM','M','W'}
        vulling = 'MWMW';
    otherwise
        error(['Could not determine the requested symmetry from vulling = ' vulling '!'])
    end % switch vulling
else
    switch vulling(1:2)
    case {'\/','/\'}
        vulling(1:2) = '\/';
    case {'-|','|-'}
        vulling(1:2) = '-|';
    case {'x+','+x'}
        vulling(1:2) = 'x+';
    case {'sz','zs'}
        vulling(1:2) = 'sz';
    case {'<>','><'}
        vulling(1:2) = '<>';
    case {'MW','WM'}
        vulling(1:2) = 'MW';
    otherwise
        error(['Could not determine the requested symmetry from vulling(1:2) = ' vulling(1:2) '!'])
    end % switch vulling(1:2)
    if length(vulling)==3
        if vulling(3) ~= vulling(2) vulling(4)=vulling(2); else vulling(4)=vulling(1); end
    end
end % if length(vulling)<=2 of >2
% vanaf nu is vulling(1:2) '\/', '-|', 'x+', 'sz', '<>' of 'MW', vulling(3:4) de in te vullen karakters,

% checken dat trafos van de goede vorm is.
trafos = reshape(trafos,1,prod(size(trafos)));
if ischar(trafos)
    invoertrafos=trafos;
    trafos = zeros(1,7);
    trafos(1) = length(find(invoertrafos=='x'));
    trafos(2) = length(find(invoertrafos=='Y'));
    trafos(3) = length(find(invoertrafos=='X'));
    trafos(4) = length(find(invoertrafos=='y'));
    trafos(5) = length(find(invoertrafos=='s'));
    trafos(6) = length(find(invoertrafos=='d'));
    trafos(7) = length(find(invoertrafos=='f'));
    clear invoertrafos;
end % if trafos string -> getallen
if length(trafos) < 7 defaulttrafos = [1 0 0 1 1 1 0]; trafos = [trafos defaulttrafos(length(trafos)+1,end)]; end % trafos zo nodig aanvullen met DEFAULT-waarden.
trafos = floor(trafos(1:7));
if ismember(vulling(1:2), {'<>', 'MW'})
    if ismember(mod(trafos(5),5),[3 4]) trafos(5) = 0; end % Voor vulling = '<>' of 'MW' wordt geen diagonale spiegeling toegestaan.
	if mod(trafos(6),4) trafos(6) = 2; end % Voor vulling = '<>' of 'MW' wordt alleen puntspiegeling d^2 toegestaan.
end
% Vanaf nu is trafos een 1x7 array met integers.
% Waarbij trafos(5)=0,1,2 en trafos(6)=0,2 in het geval dat vulling = '<>' of 'MW'.

% checken dat uitvoer van de goede vorm is.
if length(uitvoer) < 7 uitvoer = ['stilte:' uitvoer]; end
switch uitvoer(1:7)
case {'scherm:','append:','erover:','cancel:','stilte:'}
    % Als er begonnen wordt met een van de codes dan hoeft er niets veranderd te worden.
otherwise
    % Als uitvoer alleen de bestandsnaam bevat, dan moet nog even de default-code 'stilte:' volgen.
    uitvoer = ['stilte:' uitvoer];
end
% uitvoer begint nu dus altijd met een code van 7 karakters.
clear fid % zeker weten dat fid nog niet bestaat
if length(uitvoer) > 7 % Er is een bestandsnaam opgegeven.
    if exist(uitvoer(8:end), 'file') % bestand bestaat al.
        % Wat moet er gedaan worden?
        switch uitvoer(1:7)
        case {'scherm:', 'stilte:'} % bij de standaard-instellingen wordt nu gevraagd wat te doen.
            overschrijf = menu(sprintf('Het bestand %s bestaat al. Wat wilt u doen?', uitvoer(8:end)),...
                'Het bestand overschrijven.', 'Verder schrijven achter dit bestand.',...
                'Verder gaan met alleen uitvoer naar het scherm', 'Annuleren');
        case 'erover:'
            overschrijf = 1;
        case 'append:'
            overschrijf = 2;
        case 'cancel:'
            overschrijf = 4;
        otherwise
            error(['Een onbekende uitvoercode uitvoer(1:7) = ' uitvoer(1:7) '. Dit zou niet moeten kunnen!'])
        end % switch uitvoer-code
        % Bestand overschrijven, aanvullen of annuleren.
        switch overschrijf
        case 1
            [fid, foutmelding] = fopen(uitvoer(8:end),'w');
        case 2
            [fid, foutmelding] = fopen(uitvoer(8:end),'a');
            fprintf(fid, '\n\n====================================================== \n  Nieuwe uitvoer van schrijfpatronen.m toegevoegd \n====================================================== \n\n');
        case 3
            clear fid
            uitvoer = 'scherm:';
        case 4
            return;
        end % switch overschrijf
    else
        % Bestand bestaat nog niet.
        [fid, foutmelding] = fopen(uitvoer(8:end),'w');
    end % if bestand bestaat al wel of niet

    if exist('fid', 'var') & fid == -1 % Als het bestand niet geopend kan worden.
        % Wat moet er gedaan worden?
        switch uitvoer(1:7)
        case {'scherm:', 'stilte:'} % bij de standaard-instellingen wordt nu gevraagd wat te doen.
            foutkeuze = menu(sprintf('Het bestand %s kan niet geopend worden:\n %s\n\nWat wilt u doen?',uitvoer(8:end),foutmelding),...
                'Verder gaan met alleen uitvoer naar het scherm', 'Annuleren');
        case {'erover:', 'append:', 'cancel:'} % Bij onderdrukte interactie altijd annuleren.
            warning(sprintf('Het bestand %s kan niet geopend worden:\n %s\n\nUitvoer voor m x n = %i x %i, uitbreiding %i x hor, %i x vert, vulling = ''%s'', is geannuleerd!',uitvoer(8:end),foutmelding,m,n,uitbr(1),uitbr(2),vulling))
            return;
        otherwise
            error(['Een onbekende uitvoercode uitvoer(1:7) = ' uitvoer(1:7) '. Dit zou niet moeten kunnen!'])
        end % switch uitvoer-code
        % Dan maar alleen naar het scherm of annuleren.
        switch foutkeuze
        case 1
            clear fid
            uitvoer = 'scherm:';
        case 2
            return;
        end % switch foutkeuze
	end % if fout bij openen
end % if bestandsnaam opgegeven
uitvoer = isequal(uitvoer(1:7), 'scherm:'); % vanaf nu is uitvoer een logical die wel/niet uitvoer naar scherm bepaalt.


% NU GAAN WE DAADWERKELIJK DE PATRONEN BEPALEN.
aantal = aantalpatronen(m,n,vulling(1:2),trafos);
patterns = bepaalpatronen(m,n,vulling(1:2),trafos);


% DAN VOLGT NU DE UITVOER
% Eerst de info-regels:
if trafos == 0
    % als alle symmetrien niet meedoen (dan zijn alle 2^(m*n) patronen verschillend)
    tekst = sprintf('Hier volgen de %i onafhankelijke patronen (zonder rekening te houden met symmetrien)\n', aantal);
else
    tekst = sprintf('Hier volgen de %i onafhankelijke patronen ten opzichte van de symmetrien\n', aantal);
    symms = '';
        symms{1} = '   - ''x''  = x^%i y^%i, als ''verticale'' translatie (%i hokjes omhoog, %i naar rechts)\n';
        symms{2} = '   - ''y''  = x^%i y^%i, als ''horizontale'' translatie (%i hokjes omhoog, %i hokjes naar rechts)\n';
        symms{3} = '   - s    = spiegeling (in verticale as)\n';             symms{4} = '   - sdd  = spiegeling (in horizontale as)\n';
        symms{5} = '   - sddd = spiegeling (in stijgende diagonaal)\n';      symms{6} = '   - sdd  = spiegeling (in dalende diagonaal)\n';
        symms{7} = '   - d    = rotatie (kwartslag rechtsom)\n';             symms{8} = '   - dd   = rotatie over 180 graden (puntspiegeling)\n';
        symms{9} = '   - ddd  = rotatie (kwartslag linksom)\n';              symms{10}= '   - f    = ''flip-transformatie'', die de elementen in hun tegenhanger omzet.\n';
    if sum(mod(trafos(1:2),m)) tekst = [tekst, sprintf(symms{1}, trafos(1),trafos(2), trafos(1),trafos(2))]; end
    if sum(mod(trafos(3:4),n)) tekst = [tekst, sprintf(symms{2}, trafos(3),trafos(4), trafos(3),trafos(4))]; end
    if mod(trafos(5),5) tekst = [tekst, sprintf(symms{2+mod(trafos(5),5)})]; end
    if mod(trafos(6),4) tekst = [tekst, sprintf(symms{6+mod(trafos(6),4)})]; end
    if mod(trafos(7),2) tekst = [tekst, sprintf(symms{10})]; end
    clear symms
end % helemaal geen symmetrien
tekst = [tekst, sprintf('die worden gemaakt door voortzetting van patronen bestaande uit de elementen\n')];
tekst = [tekst, sprintf('''%c'' (hier weergegeven door ''%c'') en ''%c'' (hier weergegeven door ''%c'')\n', vulling(1), vulling(3),vulling(2),vulling(4))];
tekst = [tekst, sprintf('op de %i vakjes in een %i x %i rechthoek tot een patroon op het hele rooster Z^2.\n\n', m*n, m,n)];
% uitvoer naar bestand en/of scherm.
if uitvoer disp(tekst); end; if exist('fid', 'var') fwrite(fid, [tekst, sprintf('\n')]); end

% Koptekst bij de kolommen
tekst = 'Volgnr.:      binair patroon-getal                fundamentaalgebied';
if uitbr tekst = [tekst, sprintf('  periodieke uitbreiding (%i x hor en %i x vert)', uitbr(2), uitbr(1))]; end
tekst = [tekst, sprintf('\n')];
% uitvoer naar bestand en/of scherm.
if uitvoer disp(tekst); end; if exist('fid', 'var') fwrite(fid, [tekst, sprintf('\n')]); end

% Loop over alle gevonden patronen.
for i=1:length(patterns)
    % char-matrices met de patronen
    fundam = tekenpatroon(patterns(i),m,n, [1,1], vulling(3:4));
    if uitbr patroon = tekenpatroon(patterns(i),m,n, uitbr, vulling(3:4)); end

    % De eerste regel
    tekst = sprintf('%6i :  %8i = %-25s    %15s', i, patterns(i), dec2bin(patterns(i),m*n), fundam(1,:));
    if uitbr tekst = [tekst '        ' patroon(1,:)]; end; tekst = [tekst sprintf('\n')];
    % de volgende regels over het fundamentaalgebied
    for r = 2:m
        tekst = [tekst char(' '*ones(1,50)) sprintf('%15s', fundam(r,:))];
        if uitbr tekst = [tekst '        ' patroon(r,:)]; end; tekst = [tekst sprintf('\n')];
    end
    % Desgewenst over de uitbreiding
    if uitbr
        for r = m+1:uitbr(1)*m
            tekst = [tekst char(' '*ones(1,73)) patroon(r,:) sprintf('\n')];
        end
    end
    % uitvoer naar bestand en/of scherm.
	if uitvoer disp(tekst); end; if exist('fid', 'var') fwrite(fid, [tekst, sprintf('\n')]); end
end % loop r over patronen.


% TENSLOTTE ALLEEN NOG EVEN HET BESTAND SLUITEN
if exist('fid', 'var') fclose(fid); end