% ===================================================================================================================================
% Copyright notice:
% These MATLAB-routines have been developed by Guido Terra in answer to a question by Robert Nottrot raised through "www.wisfaq.nl".
% You are free to use, reproduce and distribute them for non-commercial use as long as this copyright notice is included verbatim.
% If you want to modify the code, send a copy of your changes to the author at "terra@nioz.nl".
% Moreover, you are strongly encouraged to contact me to report any bugs, mistakes and other errors.
% The routines have been written for MATLAB-release 13; at least one error is known to occur when using release 12.
% A complete set of files would contain the following 13 files:
%   aantalpatronen.m, bepaalcykels.m, cykellengtes.m,  bepaalpatronen.m, bepaalallebanen.m, bepaalbanen.m
%   equivpatronen.m, diagequivpatronen.m, tekenpatroon.m, patroongetal.m, schrijfpatronen.m
%   patronendemo.m, patronendemo.fig
% This ends the copyright notice.
% ===================================================================================================================================

% Deze matlab-functie tekent het patroon van tekens op m bij n vakjes
% dat wordt gerepresenteerd door het getal q, door het uitvoeren van een string-array met tekens.
% De binaire ontwikkeling van q bepaalt het patroon '0' <-> vulling(1) en '1' <-> vulling(2), DEFAULT '\' resp. '/'.
% De optionele parameter uitbr biedt de mogelijkheid om het patroon periodiek voort te zetten
% buiten het fundamentaalgebied van m bij n vakjes.
% Met invoer uitbr = [k l] wordt er verticaal uitgebreid tot k copies (totale hoogte k*m)
%   en horizontaal tot l copies (totale lengte l*n)
% vulling = 'bin' zorgt voor een representatie met nullen en enen in de string.
% vulling = '\/' zorgt voor de representatie met diagonalen.
% vulling = '-|' zorgt voor de representatie met diagonalen.
% vulling = 'x+' zorgt voor de representatie met kruizen.
% vulling = 'sz' zorgt voor de representatie met 's' en 'z' (er is volgens geen ascii-teken met d-symmetrie en s-antisymmetrie)
% vulling = '<>' zorgt voor de representatie met '<' en '>'.
% vulling = 'MW' zorgt voor de representatie met 'M' en 'W'.
% je kunt zelfs elke andere string geven: de nullen worden vulling(1), de enen vulling(2).
% merk op dat vulling = '01' dus hetzelfde oplevert als vulling = 'bin'.
% en vulling([2 1]) het omgekeerde patroon t.o.v. de tekens in vulling.
%
% UITVOER p is een string-array met afmetingen [k*m l*n size(q)], dus het werkt zelfs goed als q niet scalar is.

function p = tekenpatroon(q, m,n, uitbr, vulling)

% checken dat m,n twee gehele getallen zijn.
if prod([size(m), size(n)]) ~= 1 | ceil([m,n]) ~= [m,n] | m <= 0 | n <=0
    error('m and n must be positive integers!');
end
% checken dat q hierop een patroon representeert
if ceil(q) ~= q
    error('Patterns should be represented by integers q!');
%    error(['q = ' num2str(q) ' ~= ' dec2bin(q,m*n) ' does not represent a single pattern on the ' num2str(m) ' by ' num2str(n) ' rectangle!']);
end
% patroongetallen modulo maximum getal en de afmetingen onthouden.
q = mod(q,2^(m*n)); afmetingen = size(q);

% DEFAULT-waarde voor vulling en uitbr
if nargin < 5
    vulling = '\/';
if nargin < 4
    uitbr = [1,1];
end;end
if isempty(uitbr) % uitbr = [] is placeholder
    uitbr = [1,1];
end
% checken dat uitbr van de goede vorm is
if prod(size(uitbr)) ~= 2 | ~(ceil(uitbr) == uitbr) | ~(uitbr > [0,0])
    error('Extension parameter uitbr is not of the form [k,l], with k,l positive integers!')
end

% Nu gaan we het patroon op het fundamentaalgebied vormen
q = dec2bin(reshape(q,1,prod(afmetingen)),m*n); % de binaire representatie
if ~isequal(vulling, 'bin') % als vulling == 'bin' dan houden we de binaire representatie aan.
    if vulling(1) ~= '1'
        q(q=='0') = vulling(1); q(q=='1') = vulling(2); % de patroonvormen (bijvoorbeeld diagonalen) intekenen
    elseif vulling(2) ~= '0'
        q(q=='1') = vulling(2); q(q=='0') = vulling(1); % al de nullen op '1' worden gezet, dan moet je eerst de enen veranderen.
    else
        tq = (q=='0'); q(q=='1') = vulling(2); q(tq) = vulling(1); % dit geval moet wel het omwisselen van enen en nullen zijn.
    end
end
q = reshape(q',[m,n,afmetingen]); % in de rechthoek zetten

% Tenslotte het patroon met uitbreiding tekenen
p = q; % q is het patroon op het fundamentaalgebied
for t=2:uitbr(1) p = [p;q]; end % verticaal uitbreiden met k-1 copies.
q = p; % maak van q de basiskolom
for t=2:uitbr(2) p = [p,q]; end % horizontaal uitbreiden met l-1 copies.